package uremote;

import java.io.Serializable;
import java.util.Hashtable;

/**
 * A class that descibes the components offered by a device.
 *
 * @author Greg Hackmann
 */
public class AdvertisementInfo implements Serializable
{
	private String deviceName, deviceID, deviceIcon;
	private Hashtable functionToComponent;
	
	/**
	 * Instantiate an advertisement of this device's features.
	 *
	 * @param deviceName the name of the device
	 * (see {@link DeviceDefinition#getName()})
	 * @param deviceID the ID of the device
	 * (see {@link DeviceDefinition#getID()})
	 * @param deviceIcon the device's icon
	 * (see {@link DeviceDefinition#getIcon()})
	 * @param functionToComponent a {@link java.util.Hashtable} mapping a list
	 * of functions (see {@link DeviceDefinition#getFunctions()}) to
	 * their corresponding {@link ComponentAdvertisement}s
	 */
	public AdvertisementInfo(String deviceName, String deviceID, String deviceIcon, Hashtable functionToComponent)
	{
		this.deviceName = deviceName;
		this.deviceID = deviceID;
		this.deviceIcon = deviceIcon;
		this.functionToComponent = functionToComponent;
	}
	
	/**
	 * Get the device's ID.
	 *
	 * @return the device's ID
	 */
	public String getDeviceID() { return deviceID; }
	/**
	 * Get the device's name.
	 *
	 * @return the device's name
	 */
	public String getDeviceName() { return deviceName; }
	/**
	 * Get the device's icon.
	 *
	 * @return the device's icon
	 */
	public String getDeviceIcon() { return deviceIcon; }
	/**
	 * Get the Hashtable mapping the advertised functions to their
	 * advertisements.
	 *
	 * @return the function-to-advertisement map
	 */
	public Hashtable getAllFunctionInfo() { return functionToComponent; }
	/**
	 * Get the {@link ComponentAdvertisement} for the given function.
	 *
	 * @param functionName the name of the function to look up
	 * @return the function's advertisement, or null if the function is
	 * undefined
	 */
	public ComponentAdvertisement getFunctionComponent(String functionName)
	{
		return (ComponentAdvertisement)functionToComponent.get(functionName);
	}
}
