package uremote;

/**
 * A class that advertises a button's appearance.
 *
 * @author Greg Hackmann
 */
public class ButtonAdvertisement extends ComponentAdvertisement
{
	private String pic;
	private boolean toggle;
	
	/**
	 * Advertise a button of the default size (1 x 1).
	 *
	 * @param functionName the ID of this button
	 * @param toggle true if the button has an 'on' and 'off' state, false otherwise
	 * @param pic the name of the icon to represent this button
	 * (see {@link uremote.swt.ui.RemoteIconManager#iconFilename(String)})
	 * @param x the X location of the button, in grid squares
	 * @param y the Y location of the button, in grid squares
	 */
	public ButtonAdvertisement(String functionName, boolean toggle, String pic, int x, int y)
	{
		this(functionName, toggle, pic, x, y, 1, 1);
	}
	
	/**
	 * Advertise a button of the given size.
	 *
	 * @param functionName the ID of this button
	 * @param toggle true if the button has an 'on' and 'off' state, false otherwise
	 * @param pic the name of the icon to represent this button
	 * (see {@link uremote.swt.ui.RemoteIconManager#iconFilename(String)})
	 * @param x the X location of the button, in grid squares
	 * @param y the Y location of the button, in grid squares
	 * @param width the width of the button, in grid squares
	 * @param height the height of the button, in grid squares
	 */
	public ButtonAdvertisement(String functionName, boolean toggle, String pic, int x, int y, int width, int height)
	{
		super(functionName, x, y, width, height);
		this.toggle = toggle;
		this.pic = pic;
	}
	
	/**
	 * Determine whether or not this button has an 'on' and 'off' state
	 *
	 * @return true if it does, false if not
	 */
	public boolean isToggle() { return toggle; }

	/**
	 * Get the name of this button's icon.
	 *
	 * @return the icon's name
	 */
	public String getIcon() { return pic; }
}
