package uremote;

import java.io.Serializable;

/**
 * A class that advertises a component's appearance.
 *
 * @author Greg Hackmann
 */
public abstract class ComponentAdvertisement implements Serializable
{
	/** The width of the component, in grid squares. */
	protected int width;
	/** The height of the component, in grid squares. */
	protected int height;
	/** The X location of the component, in grid squares. */
	protected int x;
	/** The Y location of the component, in grid squares. */
	protected int y;
	/** The ID of the component. */
	protected String functionName;
	
	/**
	 * Advertise a component of the default size (1 x 1).
	 *
	 * @param functionName the ID of this component
	 * @param x the X location of the component, in grid squares
	 * @param y the Y location of the component, in grid squares
	 */
	public ComponentAdvertisement(String functionName, int x, int y)
	{
		this(functionName, x, y, 1, 1);
	}
	
	/**
	 * Advertise a component of the given size.
	 *
	 * @param functionName the ID of this component
	 * @param x the X location of the component, in grid squares
	 * @param y the Y location of the component, in grid squares
	 * @param width the width of the component, in grid squares
	 * @param height the height of the component, in grid squares
	 */
	public ComponentAdvertisement(String functionName, int x, int y, int width, int height)
	{
		this.functionName = functionName;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
	}
	
	/**
	 * Get the component's function ID.
	 *
	 * @return the component's function ID
	 */
	public String getFunctionName() { return functionName; }

	/**
	 * Get the component's X location.
	 *
	 * @return the component's X location
	 */
	public int getX() { return x; }
	
	/**
	 * Get the component's Y location.
	 *
	 * @return the component's Y location
	 */
	public int getY() { return y; }
	
	/**
	 * Get the component's width.
	 *
	 * @return the component's width
	 */
	public int getWidth() { return width; }
	
	/**
	 * Get the component's height.
	 *
	 * @return the component's height
	 */
	public int getHeight() { return height; }
}

