package uremote;

import java.io.Serializable;

/**
 * A class that describes the state of a component.
 *
 * @author Greg Hackmann
 */
public abstract class ComponentState implements Serializable
{
	/** A flag specifying whether or not the component is enabled. */
	protected boolean enabled;
	
	/**
	 * Instantiate a description of a component's state.
	 *
	 * @param enabled true if the component is enabled, false if it is disabled
	 */
	public ComponentState(boolean enabled)
	{
		this.enabled = enabled;
	}
	
	/**
	 * Determine whether or not the component is currently enabled.
	 *
	 * @return true if the component is enabled, false if it is disabled
	 */
	public boolean isEnabled() { return enabled; }
}
