package uremote;

import java.io.Serializable;
import java.util.Hashtable;

/**
 * A class that provides the "help" text for a device's advertised functions.
 *
 * @author Greg Hackmann
 */
public class HelpInfo implements Serializable
{
	private String deviceID;
	private Hashtable functionToHelp;
	
	/**
	 * Provide help text for each of the device's features.
	 *
	 * @param deviceID the ID of the device
	 * (see {@link DeviceDefinition#getID()})
	 * @param functionToHelp a {@link java.util.Hashtable} mapping a list
	 * of functions (see {@link DeviceDefinition#getFunctions()}) to
	 * their corresponding "help" text (in a String)
	 */
	public HelpInfo(String deviceID, Hashtable functionToHelp)
	{
		this.deviceID = deviceID;
		this.functionToHelp = functionToHelp;
	}
	
	/**
	 * Get the device's ID.
	 *
	 * @return the device's ID
	 */
	public String getDeviceID() { return deviceID; }
	/**
	 * Get the Hashtable mapping the advertised functions to their
	 * "help" text.
	 *
	 * @return the function-to-help map
	 */
	public Hashtable getAllHelpInfo() { return functionToHelp; }
	/**
	 * Get the "help" text for the given function.
	 *
	 * @param functionName the name of the function to look up
	 * @return the function's "help" text, or null if the function is
	 * undefined
	 */
	public String getFunctionHelp(String functionName)
	{
		return (String)functionToHelp.get(functionName);
	}
}
