package uremote;

import java.awt.image.BufferedImage;

/**
 * A class that describes an image's state.
 *
 * @author Greg Hackmann
 */
public class ImageState extends ComponentState
{
	private int [] data;
	private int width, height;
	private String caption;
	
	/**
	 * Instantiate a description of an image's state.
	 *
	 * @param image the {@link java.awt.Image} to be displayed on graphical remotes
	 * @param caption the caption to be displayed on non-graphical remotes
	 */
	public ImageState(BufferedImage image, String caption)
	{
		super(true);
		this.caption = caption;
		width = image.getWidth();
		height = image.getHeight();

		data = new int[width * height];
		image.getRGB(0, 0, width, height, data, 0, width);
		/* We have to store just the RGB data, since BufferedImages
		   aren't serializable */
	}
	
	/**
	 * Get the image's caption.
	 *
	 * @return the image's caption
	 */
	public String getCaption() { return caption; }

	/**
	 * Get the image's data.
	 * 
	 * @return the image's data as an AWT {@link java.awt.image.BufferedImage}
	 */
	public BufferedImage getImage()
	{
		final BufferedImage returnMe = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		returnMe.setRGB(0, 0, width, height, data, 0, width);
		/* Put the image data back into a new BufferedImage */
		
		return returnMe;
	}
}
