package uremote;

/**
 * A class that advertises a slider's appearance.
 *
 * @author Greg Hackmann
 */
public class SliderAdvertisement extends ComponentAdvertisement
{
	private int min, max;
	
	/**
	 * Advertise a slider of the default size (1 x 1).
	 *
	 * @param functionName the ID of this slider
	 * @param min the minimal value of the slider
	 * @param max the maximal value of the slider
	 * @param x the X location of the slider, in grid squares
	 * @param y the Y location of the slider, in grid squares
	 */
	public SliderAdvertisement(String functionName, int min, int max, int x, int y)
	{
		this(functionName, min, max, x, y, 1, 1);
	}
	
	/**
	 * Advertise a slider of the given size.
	 *
	 * @param functionName the ID of this slider
	 * @param min the minimal value of the slider
	 * @param max the maximal value of the slider
	 * @param x the X location of the slider, in grid squares
	 * @param y the Y location of the slider, in grid squares
	 * @param width the width of the slider, in grid squares
	 * @param height the height of the slider, in grid squares
	 */
	public SliderAdvertisement(String functionName, int min, int max, int x, int y, int width, int height)
	{
		super(functionName, x, y, width, height);
		this.min = min;
		this.max = max;
	}
	
	/**
	 * Get the minimal value allowed by the slider.
	 *
	 * @return the slider's minimal value
	 */
	public int getMin() { return min; }
	/**
	 * Get the maximal value allowed by the slider.
	 *
	 * @return the slider's maximal value
	 */
	public int getMax() { return max; }
}
