package uremote;

import java.io.Serializable;
import java.util.Hashtable;

/**
 * A class that descibes the state of a device's functions.
 *
 * @author Greg Hackmann
 */
public class StateInfo implements Serializable
{
	private String deviceID;
	private Hashtable functionToState;
	
	/**
	 * Instantiate a description of this device's state.
	 *
	 * @param deviceID the ID of the device
	 * (see {@link DeviceDefinition#getID()})
	 * @param functionToState a {@link java.util.Hashtable} mapping a list
	 * of functions (see {@link DeviceDefinition#getFunctions()}) to
	 * their corresponding {@link ComponentState}s
	 */
	public StateInfo(String deviceID, Hashtable functionToState)
	{
		this.deviceID = deviceID;
		this.functionToState = functionToState;
	}
	
	/**
	 * Get the device's ID.
	 *
	 * @return the device's ID
	 */
	public String getDeviceID() { return deviceID; }
	
	/**
	 * Get the Hashtable mapping the device's functions to their
	 * states.
	 *
	 * @return the function-to-state map
	 */
	public Hashtable getFunctionToState() { return functionToState; }
}
