package uremote.awt;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import uremote.awt.ui.*;
import uremote.*;
import eLights.*;
import limone.*;

/**
 * A helper class used to manage the universal remote control's GUI.
 *
 * @author Greg Hackmann
 */
public class AWTGUIManager implements URemoteGUIManager
{
	private Hashtable pageTable;
	private RemoteDisplay display;
	private URemoteAgent agent;
	
	public AWTGUIManager(URemoteAgent agent)
	{
		this.agent = agent;
		pageTable = new Hashtable();
		
		startDisplay();
	}
	
	private void startDisplay()
	{
		display = new RemoteDisplay();
		
		new Thread()
		{
			public void run()
			{
				display.open();
			}
		}.start();
	}
	
	public void addDevice(final AgentID id, final AdvertisementInfo ad, final HelpInfo help)
	{
		Hashtable buttonTable = ad.getAllFunctionInfo();
		Enumeration enum = buttonTable.elements();
		Vector components = new Vector();
		String deviceName = ad.getDeviceName();
		String deviceID = ad.getDeviceID();
		
		RemotePage page = display.makePage(deviceName, deviceID);
		page.setText(deviceName);

		while(enum.hasMoreElements())
		{
			final ComponentAdvertisement current = (ComponentAdvertisement)enum.nextElement();
			boolean viewable = true;

			if(current instanceof ButtonAdvertisement)
			{
				ButtonAdvertisement buttonAd = (ButtonAdvertisement)current;
				final RemoteButton button = page.makeButton(current.getFunctionName(), current.getX(), current.getY(), current.getWidth(), current.getHeight());
				
				//String buttonLabel = RemoteIconManager.iconLabel(buttonAd.getIcon());
				//if(buttonLabel == null) buttonLabel = current.getFunctionName();
				//button.setText(buttonLabel);
				
/*				String buttonIcon = RemoteIconManager.iconFilename(buttonAd.getIcon());
				if(buttonIcon != null)
				{
					button.setImage(display.loadImage(buttonIcon + ".jpg"));
					button.setDisabledImage(display.loadImage(buttonIcon + "-disabled.jpg"));
					button.setLitImage(display.loadImage(buttonIcon + "-lit.jpg"));
				}
				else
				{*/
					String buttonLabel = RemoteIconManager.iconLabel(buttonAd.getIcon());
					if(buttonLabel == null) buttonLabel = current.getFunctionName();
				    button.setText(buttonLabel);
				/*}*/
				
				button.setViewable(viewable);
				String helpText = help.getFunctionHelp(current.getFunctionName());
				if(helpText != null) button.setHelpText(helpText);
				
				button.getButton().addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						buttonClicked(button, current.getFunctionName(), id);
					}
				});
			}
			else if(current instanceof SliderAdvertisement)
			{
				SliderAdvertisement currentAd = (SliderAdvertisement)current;
				final RemoteSlider slider = page.makeSlider(current.getFunctionName(), current.getX(), current.getY(), current.getWidth(), current.getHeight());
				String helpText = help.getFunctionHelp(current.getFunctionName());
				if(helpText != null) slider.setHelpText(helpText);
				
				slider.getSlider().setMinimum(currentAd.getMin());
				slider.getSlider().setMaximum(currentAd.getMax());
				slider.setViewable(viewable);

				slider.getSlider().addAdjustmentListener(new AdjustmentListener()
				{
					public void adjustmentValueChanged(AdjustmentEvent e)
					{
						sliderClicked(slider, current.getFunctionName(), id);
					}
				});
			}
			else if(current instanceof LabelAdvertisement)
			{
				RemoteLabel label = page.makeLabel(current.getFunctionName(), current.getX(), current.getY(), current.getWidth(), current.getHeight());
				label.setViewable(viewable);
			}
			else if(current instanceof ImageAdvertisement)
			{
				RemoteImage image = page.makeImage(current.getFunctionName(), current.getX(), current.getY(), current.getWidth(), current.getHeight());
			}
		}
		
//		page.getPanel().validate(); page.getPanel().invalidate();
//		page.show();
		display.update();
		pageTable.put(ad.getDeviceID(), page);
	}
	
	public void deleteDevice(final String id)
	{
		RemotePage removeMe = (RemotePage)pageTable.get(id);
		if(removeMe == null) return;
		
		display.removePage(removeMe);
		pageTable.remove(id);
	}
	
	private void sliderClicked(final RemoteSlider src, String functionName, final AgentID id)
	{
		new Thread()
		{
			public void run()
			{
				ActionInfo cmd = new ActionInfo(src.getFunctionName(), src.getSlider().getValue());
				ActionTuple sendMe = new ActionTuple(cmd);
				sendMe.addField(new EField("aID", agent.getAgentID()));
				
				AcquaintanceList aql = agent.getAQL();
				AgentProxy proxy = aql.getAgentProxy(id);
				if(proxy == null) return;
				
				try
				{
					proxy.out(sendMe);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}.start();
	}

	private void buttonClicked(final RemoteButton src, final String functionName, final AgentID id)
	{
		new Thread()
		{
			public void run()
			{
				ActionInfo cmd = new ActionInfo(src.getFunctionName(), true);
				ActionTuple sendMe = new ActionTuple(cmd);
				sendMe.addField(new EField("aID", agent.getAgentID()));
					
				AcquaintanceList aql = agent.getAQL();
				AgentProxy proxy = aql.getAgentProxy(id);
				if(proxy == null) return;
				
				try
				{
					proxy.out(sendMe);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}.start();
	}
	
	/**
	 * Update the GUI to reflect a device's new state.
	 *
	 * @param info the state information of the device to be updated
	 */
	public void update(StateInfo info)
	{
		RemotePage page = (RemotePage)pageTable.get(info.getDeviceID());
		if(page == null) return;
		
		Hashtable stateTable = info.getFunctionToState();
		Vector kids = page.getPanel().getChildren();
		
		for(Enumeration enum = kids.elements(); enum.hasMoreElements();)
		{
			RemoteComponent current = (RemoteComponent)enum.nextElement();
			if(current instanceof RemoteBlank) continue;

			ComponentState currentState = (ComponentState)stateTable.get(current.getFunctionName());
			if(currentState == null) continue;
			
			if(current instanceof RemoteButton)
			{
				RemoteButton button = (RemoteButton)current;
				ButtonState buttonState = (ButtonState)currentState;
				
				boolean enabled = buttonState.isEnabled();
				if(button.isClickable() != enabled)
				button.setClickable(enabled);
					
				boolean lit = buttonState.isLit();
				if(button.isLit() != lit)
					button.setLit(lit);
			}
			else if(current instanceof RemoteSlider)
			{
				RemoteSlider slider = (RemoteSlider)current;
				Scrollbar child = slider.getSlider();
				SliderState sliderState = (SliderState)currentState;

				boolean enabled = sliderState.isEnabled();
				if(slider.isClickable() != enabled)
					slider.setClickable(enabled);
					
				int value = sliderState.getValue();
				if(child.getValue() != value)
					child.setValue(value);
					
				String text = sliderState.getLabel();
				if(!text.equals(slider.getText()))
					slider.setText(text);
			}
			else if(current instanceof RemoteLabel)
			{
				RemoteLabel label = (RemoteLabel)current;
				LabelState labelState = (LabelState)currentState;

				boolean enabled = labelState.isEnabled();
				if(label.isClickable() != enabled)
					label.setClickable(enabled);

				String text = labelState.getText();
				if(!text.equals(label.getText()))
					label.setText(text);
			}
			else if(current instanceof RemoteImage)
			{
				RemoteImage image = (RemoteImage)current;
				ImageState imageState = (ImageState)currentState;
				
				image.setImage(imageState.getImage());
			}
		}
	}
}
