package uremote.awt.ui;

import java.awt.*;

/**
 * A class that represents an on-screen button.
 *
 * @author Greg Hackmann
 */
public class RemoteButton extends RemoteComponent
{
	private Button button;
	private static Color normalColor = new Color(239, 239, 239), litColor = Color.yellow;
	
	/**
	 * Instantiate the RemoteButton to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.awt.ui.RemotePage#makeButton}
	 * instead.
	 *
	 * @param parent the {@link java.awt.Container} that will contain this RemoteButton
	 * @param id the function name of the button
	 * @param x the X coordinate of the button, in grid squares
	 * @param y the Y coordinate of the button, in grid squares
	 * @param w the width of the button, in grid squares
	 * @param h the height of the button, in grid squares
	 */
	public RemoteButton(Container parent, String id, int x, int y, int w, int h)
	{
		super(parent, id, x, y, w, h);
		button = new Button(id);
		button.setBackground(normalColor);
		add(button);
	}
	
	public void setLit(boolean flag)
	{
		super.setLit(flag);
		button.setBackground(flag ? litColor : normalColor);
	}
	
	public void setText(String text)
	{
		super.setText(text);
		button.setLabel(text);
	}
	
	public void setClickable(boolean flag)
	{
		super.setClickable(flag);
		button.setEnabled(flag);
	}
	
	/**
	 * Get the {@link java.awt.Button} being used by this RemoteButton.
	 *
	 * @return the Button being used by this RemoteButton
	 */
	public Button getButton()
	{
		return button;
	}
}
