package uremote.awt.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 * A class representing the Universal Remote's GUI.
 *
 * @author Greg Hackmann
 */
public class RemoteDisplay
{
	private Frame frame;
	private Panel tabPanel;
	private Choice tabChoice;
	private Vector idVector;
	
	/**
	 * Instantiate, but do not show, the GUI.  The GUI should be shown using
	 * the {@link #open()} method.
	 */
	public RemoteDisplay()
	{
		frame = new Frame("Universal Remote");
		frame.setLayout(new BorderLayout());
		frame.setBackground(Color.lightGray);
		frame.setForeground(Color.black);
		
		tabPanel = new Panel(new CardLayout());
		frame.add(tabPanel);

		tabChoice = new Choice();
		tabChoice.setEnabled(false);
		tabChoice.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				String id = (String)idVector.elementAt(tabChoice.getSelectedIndex());
				((CardLayout)tabPanel.getLayout()).show(tabPanel, id);
			}
		});
		frame.add(tabChoice, BorderLayout.SOUTH);
		
		idVector = new Vector();
		
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
	}
	
	/**
	 * Update the GUI's display.  This should be called anytime a page is
	 * added or a component is resized.
	 */
	public void update()
	{
		frame.invalidate();
		frame.validate();
	}
	
	/**
	 * Create a {@link RemotePage} and add it to the list of pages.
	 *
	 * @return the created RemotePage
	 */
	public RemotePage makePage(final String name, String id)
	{
		RemotePanel panel = new RemotePanel(tabPanel, id);
		RemotePage returnMe = new RemotePage(panel);
		
		tabChoice.add(name);
		idVector.addElement(id);
		tabChoice.setEnabled(true);

		return returnMe;
	}
	
	/**
	 * Removes the specified RemotePage from the tabs.
	 *
	 * @param page the page to be removed
	 */
	public void removePage(RemotePage page)
	{
		tabPanel.remove(page.getPanel());
		int removeMe = tabChoice.getSelectedIndex();
		idVector.removeElementAt(removeMe);
		tabChoice.remove(removeMe);
		
		if(idVector.size() == 0) tabChoice.setEnabled(false);
		update();
	}

	/**
	 * Shows the GUI display.  This method blocks until the program has
	 * terminated.
	 */
	public void open()
	{
		frame.setSize(240, 300);
		frame.show();
	}
}
