package uremote.awt.ui;

import java.util.Hashtable;

/**
 * A class to manage the icons supported by the Universal Remote.
 *
 * @author Greg Hackmann
 */
public abstract class RemoteIconManager
{
	private static Hashtable labelMap;
	private static boolean initialized = false;
	
	private synchronized static void initialize()
	{
		labelMap = new Hashtable();
		labelMap.put("POWER", "Power");
		labelMap.put("MUTE", "Mute");
		labelMap.put("PLAY", "|>");
		labelMap.put("STOP", "[]");
		labelMap.put("PAUSE", "||");
		labelMap.put("RECORD", "O");
		labelMap.put("FASTFORWARD", ">>");
		labelMap.put("REWIND", "<<");
		labelMap.put("PREVIOUS", "|<");
		labelMap.put("NEXT", ">|");
		labelMap.put("ARROWLEFT", "<-");
		labelMap.put("ARROWRIGHT", "->");
		labelMap.put("ARROWUP", "^");
		labelMap.put("ARROWDOWN", "v");
		labelMap.put("ARROWENTER", "<-'");
	}
	
	private RemoteIconManager()
	{
	}
	
	/**
	 * Get the text label for the provided icon.
	 * <p>
	 * Currently, the following icons are guaranteed to be supported:
	 * "ARROWDOWN", "ARROWENTER", "ARROWLEFT", "ARROWRIGHT", "ARROWUP",
	 * "FASTFORWARD", "MUTE", "NEXT", "PAUSE", "PLAY", "POWER", "PREVIOUS",
	 * "RECORD", "REWIND", and "STOP".
	 *
	 * @param pic the icon to look up
	 * @return the icon's "base" filename, or null if the icon is undefined
	 */
	public synchronized static String iconLabel(String pic)
	{
		if(!initialized) initialize();
		return (String)labelMap.get(pic);
	}
}
