package uremote.awt.ui;

import java.awt.*;

/**
 * A class that represents on-screen text.
 *
 * @author Greg Hackmann
 */
public class RemoteLabel extends RemoteComponent
{
	private Label label;
	
	/**
	 * Instantiate the RemoteLabel to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.awt.ui.RemotePage#makeLabel}
	 * instead.
	 *
	 * @param parent the {@link java.awt.Container} that will contain this RemoteLabel
	 * @param id the function name of the label
	 * @param x the X coordinate of the label, in grid squares
	 * @param y the Y coordinate of the label, in grid squares
	 * @param w the width of the label, in grid squares
	 * @param h the height of the label, in grid squares
	 */
	public RemoteLabel(Container parent, String id, int x, int y, int w, int h)
	{
		super(parent, id, x, y, w, h);
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.gridx = x;
		constraints.gridy = y;
		constraints.gridwidth = w;
		constraints.gridheight = h;
		
		parent.add(this, constraints);
				
		label = new Label("", Label.CENTER);
		label.setBackground(parent.getBackground());
		label.setForeground(parent.getForeground());
		add(label);
		/* Set up the actual label */
	}
	
	/**
	 * Set the text of the label.
	 *
	 * @param t the text to use
	 */
	public void setText(String t)
	{
		super.setText(t);
		label.setText(t);
	}
}
