package uremote.awt.ui;

/**
 * A class representing a page to be placed in a tabbed list.
 *
 * @author Greg Hackmann
 */
public class RemotePage
{
	private RemotePanel panel;
	private String label;
	private boolean enabled = true;
	
	/**
	 * Instatitate a RemotePage from a given {@link RemotePanel}.
	 * This method is normally not used; the {@link RemoteDisplay#makePage(String, String)}
	 * method should be used instead.
	 *
	 * @param panel the panel to add to the display
	 */
	public RemotePage(RemotePanel panel)
	{
		this.panel = panel;
	}
	
	/**
	 * Get the {@link RemotePanel} contained by this page.
	 *
	 * @return the RemotePanel belonging to this page
	 */
	public RemotePanel getPanel() { return panel; }
	
	/**
	 * Set the tab's label.
	 *
	 * @param text the text to set as the label
	 */
	public void setText(String text) { label = text; }


	/**
	 * Create a new 1 x 1 {@link RemoteBlank}.
	 *
	 * @return the created RemoteBlank
	 */
	public RemoteBlank makeBlank(int x, int y)
	{
		return makeBlank(x, y, 1, 1);
	}

	/**
	 * Create a new {@link RemoteBlank} of the given size.
	 *
	 * @param w the width of the blank, in grid squares
	 * @param h the height of the blank, in grid squares
	 * @return the created RemoteBlank
	 */
	public RemoteBlank makeBlank(int x, int y, int w, int h)
	{
		return panel.makeBlank(x, y, w, h);
	}

	/**
	 * Create a new 5 x 1 {@link RemoteSlider}.
	 *
	 * @param id the function name of the slider
	 * @return the created RemoteSlider
	 */
	public RemoteSlider makeSlider(String id, int x, int y)
	{
		return makeSlider(id, x, y, 5, 1);
	}

	/**
	 * Create a new {@link RemoteSlider} of the given size.
	 *
	 * @param id the function name of the slider
	 * @param w the width of the slider, in grid squares
	 * @param h the height of the slider, in grid squares
	 * @return the created RemoteSlider
	 */
	public RemoteSlider makeSlider(String id, int x, int y, int w, int h)
	{
		return panel.makeSlider(id, x, y, w, h);
	}
		
	/**
	 * Create a new 1 x 1 {@link RemoteButton}.
	 *
	 * @param id the function name of the button
	 * @return the created RemoteButton
	 */
	public RemoteButton makeButton(String id, int x, int y)
	{
		return makeButton(id, x, y, 1, 1);
	}
	
	/**
	 * Create a new {@link RemoteButton} of the given size.
	 *
	 * @param id the function name of the button
	 * @param w the width of the button, in grid squares
	 * @param h the height of the button, in grid squares
	 * @return the created RemoteButton
	 */
	public RemoteButton makeButton(String id, int x, int y, int w, int h)
	{
		return panel.makeButton(id, x, y, w, h);
	}
	
	/**
	 * Create a new 1 x 1 {@link RemoteLabel}.
	 *
	 * @param id the function name of the label
	 * @return the created RemoteLabel
	 */
	public RemoteLabel makeLabel(String id, int x, int y)
	{
		return makeLabel(id, x, y, 1, 1);
	}
	
	/**
	 * Create a new {@link RemoteLabel} of the given size.
	 *
	 * @param id the function name of the label
	 * @param w the width of the label, in grid squares
	 * @param h the height of the label, in grid squares
	 * @return the created RemoteLabel
	 */
	public RemoteLabel makeLabel(String id, int x, int y, int w, int h)
	{
		return panel.makeLabel(id, x, y, w, h);
	}
	
	public RemoteImage makeImage(String id, int x, int y)
	{
		return makeImage(id, x, y, 1, 1);
	}

	public RemoteImage makeImage(String id, int x, int y, int w, int h)
	{
		return panel.makeImage(id, x, y, w, h);
	}
}
