package uremote.awt.ui;

import java.awt.*;
import java.util.*;


/**
 * A class representing a panel contained by a tabbed page.
 *
 * @author Greg Hackmann
 */
public class RemotePanel extends ScrollPane/* Panel*/
{
	private Container parent;
	private Panel child;
	private Vector children = new Vector(4);

	/**
	 * Instantiate a RemotePanel.  This method is normally not used; the
	 * {@link RemoteDisplay#makePage(String, String)} method should be used to make a
	 * {@link RemotePage} instead, and then the {@link RemotePage#getPanel()}
	 * method should be used to get a RemotePanel.
	 *
	 * @param parent the Container that will contain this panel
	 * @param id the ID of the device to be displayed on this panel
	 */
	public RemotePanel(Container parent, String id)
	{
		super(ScrollPane.SCROLLBARS_AS_NEEDED);
		
		child = new Panel(new GridBagLayout());
		add(child);
		child.setBackground(new Color(178, 178, 178));
		
		parent.add(this, id);
	}

	RemoteButton makeButton(String id, int x, int y, int w, int h)
	{
		RemoteButton returnMe = new RemoteButton(child, id, x, y, w, h);
		children.addElement(returnMe);
		
		return returnMe;
	}
	
	RemoteSlider makeSlider(String id, int x, int y, int w, int h)
	{
		RemoteSlider returnMe = new RemoteSlider(child, id, x, y, w, h);
		children.addElement(returnMe);

		return returnMe;
	}
	 
	RemoteLabel makeLabel(String id, int x, int y, int w, int h)
	{
		RemoteLabel returnMe = new RemoteLabel(child, id, x, y, w, h);
		children.addElement(returnMe);
		
		return returnMe;
	}
	
	RemoteBlank makeBlank(int x, int y, int w, int h)
	{
		RemoteBlank returnMe = new RemoteBlank(child, x, y, w, h);
		children.addElement(returnMe);
		
		return returnMe;
	}
	
	public RemoteImage makeImage(String id, int x, int y, int w, int h)
	{
		RemoteImage returnMe = new RemoteImage(child, id, x, y, w, h);
		children.addElement(returnMe);
		
		return returnMe;
	}

	Container getContents() { return child; }

	/**
	 * Get a {@link java.util.Vector} that contains all of this panel's
	 * RemoteComponents.
	 *
	 * @return the list of child RemoteComponents
	 */
	public Vector getChildren() { return children; }
}
