package uremote.awt.ui;

import java.awt.*;

/**
 * A class that represents an on-screen slider.
 *
 * @author Greg Hackmann
 */
public class RemoteSlider extends RemoteComponent
{
	private Scrollbar slider;
	private Label label;
	
	/**
	 * Instantiate the RemoteSlider to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.awt.ui.RemotePage#makeSlider}
	 * instead.
	 *
	 * @param parent the {@link java.awt.Container} that will contain this RemoteSlider
	 * @param id the function name of the slider
	 * @param x the X coordinate of the slider, in grid squares
	 * @param y the Y coordinate of the slider, in grid squares
	 * @param w the width of the slider, in grid squares
	 * @param h the height of the slider, in grid squares
	 */
	public RemoteSlider(Container parent, String id, int x, int y, int w, int h)
	{
		super(parent, id, x, y, w, h);
		setLayout(new GridLayout(2, 1));
		
		label = new Label("", Label.CENTER);
		label.setBackground(parent.getBackground());
		label.setForeground(parent.getForeground());
		add(label);
		
		slider = new Scrollbar(Scrollbar.HORIZONTAL);
		slider.setBackground(parent.getBackground());
		slider.setForeground(parent.getForeground());
		add(slider);
	}
	
	public void setText(String text) { super.setText(text); label.setText(text); }
	
	public void setClickable(boolean flag)
	{
		slider.setEnabled(flag);
		label.setEnabled(flag);
		super.setClickable(flag);
	}
	
	public void setViewable(boolean flag)
	{
		label.setVisible(flag);
	//	label.setEnabled(flag);
		slider.setVisible(flag);
		super.setViewable(flag);
	}

	/**
	 * Get the {@link java.awt.Scrollbar} being used by this RemoteSlider.
	 *
	 * @return the Scrollbar being used by this RemoteSlider
	 */
	public Scrollbar getSlider() { return slider; }
}

