package uremote.swt.ui;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.events.*;

/**
 * A class that represents an on-screen button.
 *
 * @author Greg Hackmann
 */
public class RemoteButton extends RemoteComponent
{
	/**
	 * Instantiate the RemoteButton to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.swt.ui.RemotePage#makeButton}
	 * instead.
	 *
	 * @param parent the Composite that will contain this RemoteButton
	 * @param id the function name of the button
	 * @param gridWidth the width of a single grid square, in pixels
	 * @param gridHeight the height of a single grid square, in pixels
	 */
	public RemoteButton(Composite parent, String id, int gridWidth, int gridHeight)
	{
		super(parent, id, gridWidth, gridHeight);
	}
	
	public void onPaint(PaintEvent e)
	{
		GC gc = e.gc;
		gc.setBackground(getParent().getBackground());
		gc.setForeground(getParent().getForeground());
		
		gc.fillRectangle(0, 0, getSize().x, getSize().y);
		/* Fill in the background */
		
		Image image = getImage();
		String label = getText();
		/* If we have an Image to draw */
		if(image != null)
		{
			int x = (getSize().x - image.getBounds().width) / 2;
			int y = (getSize().y - image.getBounds().height) / 2;
			/* Center it */
			
			gc.drawImage(image, x, y);
			/* Then draw it */
		}
		/* If we have a label to draw */
		else if(label != null)
		{
			gc.setFont(getFont());
			/* Use the default font */
			
			Point labelSize = gc.stringExtent(label);
			int x = (getSize().x - labelSize.x) / 2;
			int y = (getSize().y - labelSize.y) / 2;
			/* Center it */
			
			gc.drawString(label, x, y);
			/* Then draw it */
		}
	}
}
