package uremote.swt.ui;

import uremote.*;

import java.util.Vector;
import java.util.StringTokenizer;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.custom.*;

/**
 * A helper class to make on-screen help panels.
 *
 * @author Greg Hackmann
 */
public abstract class RemoteHelp
{
	private static Font noDeviceFont = null;
	
	private RemoteHelp() { }
	
	/**
	 * Make the "No Devices In Range" panel.
	 *
	 * @param tabs the CTabFolder to place this panel in
	 * @param display the RemoteDisplay that contains the CTabFolder
	 * @return the "No Devices In Range" panel
	 */
	public static RemotePanel makeNoDevicePage(CTabFolder tabs, RemoteDisplay display)
	{
		RemotePanel returnMe = new RemotePanel(tabs);
		Color background = display.getShell().getBackground();
		Color foreground = display.getShell().getForeground();
		
		Composite addToMe = returnMe.getContents();
		addToMe.setBackground(background);
		addToMe.setForeground(foreground);
		addToMe.setLayout(new GridLayout(1, false));
		
		Label text = new Label(addToMe, SWT.NONE);
		text.setBackground(background);
		text.setForeground(foreground);
		if(noDeviceFont == null)
		{
			Font oldFont = text.getFont();
			FontData fontData = oldFont.getFontData()[0];
			fontData.setHeight(12);
			noDeviceFont = new Font(addToMe.getDisplay(), fontData);
		}
		text.setFont(noDeviceFont);
		text.setText("There are currently no devices\n" +
					 "in range.");

		final Image backgroundImage = display.loadImage(URemoteConstants.path + "pics/misc/no-device-background.png");
		
		addToMe.setSize(addToMe.computeSize(SWT.DEFAULT, 190));
		addToMe.addPaintListener(new PaintListener()
		{
			public void paintControl(PaintEvent e)
			{
				e.gc.drawImage(backgroundImage, 0, 0);
			}
		});
		return returnMe;
	}

	/**
	 * Make the "main" help panel.
	 *
	 * @param tabs the CTabFolder to place this panel in
	 * @param display the RemoteDisplay that contains the CTabFolder
	 * @return the "main" help panel
	 */
	public static RemotePanel makeMainHelp(CTabFolder tabs, RemoteDisplay display)
	{
		RemotePanel returnMe = new RemotePanel(tabs);
		Color background = display.getShell().getBackground();
		Color foreground = display.getShell().getForeground();
		/* Get a panel to show the help page in */
		
		Composite addToMe = returnMe.getContents();
		addToMe.setBackground(background);
		addToMe.setForeground(foreground);
		addToMe.setLayout(new GridLayout(1, false));
		/* Now put a Composite in it to hold all the labels */
		
		for(int i = 0; i < 5; i++)
		{
			Composite item = new Composite(addToMe, SWT.NONE);
			item.setBackground(background);
			item.setForeground(foreground);
			item.setLayout(new GridLayout(2, false));
			/* Make a Composite to hold the help item */
			
			Label iconPart = new Label(item, SWT.NONE);
			String iconName = URemoteConstants.path + "pics/help/main-help-" + (i + 1) + ".png";
			iconPart.setImage(display.loadImage(iconName));
			/* Load the image for it */
			
			Label textPart = new Label(item, SWT.NONE);
			textPart.setBackground(background);
			textPart.setForeground(foreground);
			
			String labelText;
			if(i == 0) labelText = "Click a tab to select which\n" +
					               "device to view.";
			else if(i == 1) labelText = "Use the up and down\n" +
					                    "buttons to scroll the display\n" +
					                    "up and down.\n" +
					                    "(These buttons will not be\n" +
					                    "displayed if not needed.)";
			else if(i == 2) labelText = "Click this button to make\n" +
					                    "the grid visible or invisible.\n" +
					                    "(This button will be disabled while\n" +
					                    "in Edit mode and when the\n" +
										"Help page is being shown.)";
			else if(i == 3) labelText = "Click this button to enter or\n" +
					                    "leave Edit mode.\n" +
					                    "(This button will be disabled\n" +
								        "while the Help page is being\n" +
					                    "shown.)";
			else labelText = "Click this button to show or\n" +
					         "hide the Help page.";
			textPart.setText(labelText);
			/* Then add the text for the instructions */
		}
		
		addToMe.setSize(addToMe.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		/* Resize it, or it'll be invisible for some reason */
		return returnMe;
	}
	
	/**
	 * Make the help panel for Edit mode.
	 *
	 * @param tabs the CTabFolder to place this panel in
	 * @param display the RemoteDisplay that contains the CTabFolder
	 * @return the help panel for Edit mode
	 */
	public static RemotePanel makeEditHelp(CTabFolder tabs, RemoteDisplay display)
	{
		RemotePanel returnMe = new RemotePanel(tabs);
		Color background = display.getShell().getBackground();
		Color foreground = display.getShell().getForeground();
		
		Composite addToMe = returnMe.getContents();
		addToMe.setBackground(background);
		addToMe.setForeground(foreground);
		addToMe.setLayout(new GridLayout(1, false));
		
		for(int i = 0; i < 5; i++)
		{
			Composite item = new Composite(addToMe, SWT.NONE);
			item.setBackground(background);
			item.setForeground(foreground);
			item.setLayout(new GridLayout(2, false));
			
			Label iconPart = new Label(item, SWT.NONE);
			
			String iconName = URemoteConstants.path + "pics/help/edit-help-" + (i + 1) + ".png";
			iconPart.setImage(display.loadImage(iconName));
			
			Label textPart = new Label(item, SWT.NONE);
			textPart.setBackground(background);
			textPart.setForeground(foreground);
			
			String labelText;
			if(i == 0) labelText = "Click a button to select it.";
			else if(i == 1) labelText = "Click the button again to\n" +
										 "deselect it.";
			else if(i == 2) labelText = "Click another button to swap it\n" +
										 "with the selected button.";
			else if(i == 3) labelText = "Drag the right or bottom edge\n" +
		                                 "of the button to change its\n" +
		                                 "width or height.";
			else labelText = "Double-click on the button to\n" +
							  "hide or unhide it.";
			textPart.setText(labelText);
			
			item.setSize(item.getSize().x, 48);
		}
		
		addToMe.setSize(addToMe.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		return returnMe;
	}
	
	/**
	 * Make the help panel for a given {@link uremote.swt.ui.RemotePage}.
	 *
	 * @param tabs the CTabFolder to place this panel in
	 * @param page the RemotePage to make the help page from
	 * @return the help panel for this RemotePage
	 */
	public static RemotePanel makePageHelp(CTabFolder tabs, RemotePage page)
	{
		RemotePanel returnMe = new RemotePanel(tabs);
		Composite addToMe = returnMe.getContents();
		addToMe.setBackground(tabs.getBackground());
		addToMe.setLayout(new GridLayout(1, false));
		
		Vector kids = page.getPanel().getComponents();
		for(int i = 0; i < kids.size(); i++)
		{
			RemoteComponent current = (RemoteComponent)kids.elementAt(i);
			/* Get the current component */
			
			String helpText = current.getHelpText();
			if(helpText == null || helpText.equals("")) continue;
			/* Get its help text, and give up if it doesn't have one */
			
			Image helpImage = current.getImage();
			/* Get its image too */
			
			Composite helpItem = new Composite(addToMe, SWT.NONE);
			helpItem.setBackground(addToMe.getBackground());
			helpItem.setForeground(addToMe.getForeground());
			helpItem.setLayout(new GridLayout(2, false));
			/* Make a Composite to hold the help item */
			
			Label iconPart = new Label(helpItem, SWT.NONE);
			if(helpImage != null)
			{
				Image scaledImage = current.getIconImage();
				if(scaledImage == null) scaledImage = new Image(tabs.getDisplay(), helpImage.getImageData().scaledTo(32, 32));
				iconPart.setImage(scaledImage);
			}
			/* Use the icon image if it has one, or make our own if it doesn't */
			
			Label textPart = new Label(helpItem, SWT.WRAP);
			GC gc = new GC(tabs);
			
			textPart.setBackground(addToMe.getBackground());
			textPart.setForeground(addToMe.getForeground());
			/* Make a Label to hold the text part of the item */
			
			String wrappedText = new String();
			String currentLine = new String();
			StringTokenizer tokenizer = new StringTokenizer(helpText);
			
			while(tokenizer.hasMoreTokens())
			{
				String token = tokenizer.nextToken();
				if(gc.textExtent(currentLine + token).x > 180)
				{
					if(!wrappedText.equals("")) wrappedText += '\n';
					currentLine = new String();
				}
				
				wrappedText += token + ' ';
				currentLine += token + ' ';
			}
			/* Since SWT won't wrap the label for some reason, we do it outselves */
			
			textPart.setText(wrappedText);
		}
		
		addToMe.setSize(addToMe.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		return returnMe;
	}
}

