package uremote.swt.ui;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.*;

/**
 * A class that represents on-screen text.
 *
 * @author Greg Hackmann
 */
public class RemoteLabel extends RemoteComponent
{
	private Label label;
	private Color normalColor, disabledColor;
	
	/**
	 * Instantiate the RemoteLabel to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.swt.ui.RemotePage#makeLabel}
	 * instead.
	 *
	 * @param parent the Composite that will contain this RemoteLabel
	 * @param id the function name of the label
	 * @param gridWidth the width of a single grid square, in pixels
	 * @param gridHeight the height of a single grid square, in pixels
	 */
	public RemoteLabel(Composite parent, String id, int gridWidth, int gridHeight)
	{
		super(parent, id, gridWidth, gridHeight);
		
		normalColor = parent.getForeground();
		disabledColor = new Color(getDisplay(), 128, 128, 128);
		/* Make colors for both states */
		
		label = new Label(this, SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setBackground(parent.getBackground());
		label.setForeground(parent.getForeground());
		/* Set up the actual label */
		
		label.addMouseListener(this);
		label.addPaintListener(this);
		setClickSource(label);
		/* The label actually takes up the whole Composite, so we need to be
		   clued in to people trying to click on it */
		
		redraw();
		/* The label's also really picky about being redrawn for some reason */
		
		addDisposeListener(new DisposeListener()
		{
			public void widgetDisposed(DisposeEvent e)
			{
				if(label != null) label.dispose();
			}
		});
	}
	
	/**
	 * Set the text of the label.
	 *
	 * @param t the text to use
	 */
	public void setText(String t)
	{
		super.setText(t);
		label.setText(t);
	}
	
	/* In each of these cases, the label won't redraw automatically,
	   so we force it to */
	
	public void setEditing(boolean flag)
	{
		super.setEditing(flag);
		label.redraw();
	}
	
	public void setHighlighted(boolean flag)
	{
		super.setHighlighted(flag);
		label.redraw();
	}
		
	
	public void setViewable(boolean flag)
	{
		super.setViewable(flag);
		label.setForeground(flag ? normalColor : disabledColor);
		/* Windows CE doesn't like disabling labels, so we just
		   change its color */
	}
	
	public void setGridVisible(boolean flag)
	{
		super.setGridVisible(flag);
		label.redraw();
	}
	
	public void onPaint(PaintEvent e)
	{
	//	label.redraw();
	}
}
