package uremote.swt.ui;

import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * A class that represents an on-screen slider.
 *
 * @author Greg Hackmann
 */
public class RemoteSlider extends RemoteComponent
{
	private Slider slider;
	private Label label;
	private Color enabledColor, disabledColor;
	
	/**
	 * Instantiate the RemoteSlider to the given size.  This constructor should
	 * not normally be called by users; use {@link uremote.swt.ui.RemotePage#makeSlider}
	 * instead.
	 *
	 * @param parent the Composite that will contain this RemoteSlider
	 * @param id the function name of the label
	 * @param gridWidth the width of a single grid square, in pixels
	 * @param gridHeight the height of a single grid square, in pixels
	 */
	public RemoteSlider(Composite parent, String id, int gridWidth, int gridHeight)
	{
		super(parent, id, gridWidth, gridHeight);
		setLayout(new GridLayout(1, true));
//		parent.setLayoutData(data);
		
		enabledColor = parent.getForeground();
		disabledColor = new Color(getDisplay(), 128, 128, 128);
		
		label = new Label(this, SWT.CENTER);
		label.setBackground(parent.getBackground());
		label.setForeground(parent.getForeground());
		label.setLayoutData(new GridData(GridData.FILL_BOTH));
		label.addMouseListener(this);
		/* Make the label part */
		
		slider = new Slider(this, SWT.PUSH);
//		slider.setBackground(parent.getBackground());
//		slider.setForeground(parent.getForeground());
		slider.setLayoutData(new GridData(GridData.FILL_BOTH));
		slider.addMouseListener(this);
		/* Make the actual slider part */
		
		addDisposeListener(new DisposeListener()
		{
			public void widgetDisposed(DisposeEvent e)
			{
				if(slider != null) slider.dispose();
				if(label != null) label.dispose();
			}
		});
		/* Dispose both parts when we're done */
	}
	
	public void onPaint(PaintEvent e)
	{
		GC gc = e.gc;
		gc.setBackground(getParent().getBackground());
		gc.fillRectangle(0, 0, getSize().x, getSize().y);
		
		slider.redraw();
		label.redraw();
		/* Mac OS X doesn't always redraw the kids, so we have to do it
		   ourselves to get rid of some graphical glitches */
	}
	
	public void setImage(Image img) { super.setImage(img); label.setImage(img); }
	public void setText(String text) { super.setText(text); label.setText(text); }
	
	public void setClickable(boolean flag)
	{
		slider.setEnabled(flag);
		super.setClickable(flag);
	}
	
	public void setEditing(boolean flag)
	{
		slider.setEnabled(!flag);
		super.setEditing(flag);
	}
	
	public void setViewable(boolean flag)
	{
		label.setForeground(flag ? enabledColor : disabledColor);
		slider.setVisible(flag);
		super.setViewable(flag);
	}

	/**
	 * Get the {@link org.eclipse.swt.widgets.Slider} being used by this
	 * RemoteSlider.
	 *
	 * @return the Slider being used by this RemoteSlider
	 */
	public Slider getSlider() { return slider; }
}

