#include "JWinSpool.h"
#include <windows.h>
#include <winspool.h>

static JOB_INFO_1 * get_job_info(jint printer, jint job_id);
static PRINTER_INFO_2 * get_printer_info(jint printer);

JNIEXPORT jstring JNICALL Java_jwinspool_JWinSpool_getDefaultPrinter(JNIEnv * env, jclass class)
{
	char * buffer = NULL;
	DWORD length;
	HMODULE winspool;
	FARPROC GetDefaultPrinter;
	
	winspool = LoadLibrary("winspool.drv");
	if(winspool == NULL) return NULL;
	
	GetDefaultPrinter = GetProcAddress(winspool, "GetDefaultPrinterA");
	if(GetDefaultPrinter == NULL)
	{
		FreeLibrary(winspool);
		return NULL;
	}
	
	GetDefaultPrinter(NULL, &length);
	buffer = (char *)malloc(length);
	if(buffer == NULL)
	{
		FreeLibrary(winspool);
		return NULL;
	}

	if(!GetDefaultPrinter(buffer, &length))
	{
		FreeLibrary(winspool);
		return NULL;
	}
	jstring return_me = (*env)->NewStringUTF(env, buffer);

	FreeLibrary(winspool);
	free(buffer);
	return return_me;	
}

JNIEXPORT jint JNICALL Java_jwinspool_JWinSpool_openPrinter(JNIEnv * env, jclass class, jstring name)
{
	HANDLE handle;
	const char * name_string = (*env)->GetStringUTFChars(env, name, 0);
	
	if(!OpenPrinter((char *)name_string, &handle, NULL)) return 0;

	(*env)->ReleaseStringUTFChars(env, name, name_string);
	return (jint)handle;
}

JNIEXPORT jboolean JNICALL Java_jwinspool_JWinSpool_closePrinter(JNIEnv * env, jclass class, jint printer)
{
	return ClosePrinter((HANDLE)printer);
}

JNIEXPORT jint JNICALL Java_jwinspool_JWinSpool_getJobID(JNIEnv * env, jclass class, jint printer, jint job_num)
{
	JOB_INFO_1 * buffer = NULL;
	DWORD length, num_returned;

	EnumJobs((HANDLE)printer, job_num, 1, 1, NULL, 0, &length, &num_returned);
	if(length == 0) return 0;
	buffer = (JOB_INFO_1 *)malloc(length);

	if(!EnumJobs((HANDLE)printer, job_num, 1, 1, (char *)buffer, length, &length, &num_returned)) return 0;
	jint return_me = buffer[0].JobId;
		
	free(buffer);
	return return_me;
}

static JOB_INFO_1 * get_job_info(jint printer, jint job_id)
{
	JOB_INFO_1 * buffer = NULL;
	DWORD length;
	
	GetJob((HANDLE)printer, job_id, 1, NULL, 0, &length);
	if(length == 0) return NULL;
	buffer = (JOB_INFO_1 *)malloc(length);

	if(!GetJob((HANDLE)printer, job_id, 1, (char *)buffer, length, &length)) return NULL;
	return buffer;
}

JNIEXPORT jstring JNICALL Java_jwinspool_JWinSpool_getJobName(JNIEnv * env, jclass class, jint printer, jint job_id)
{
	JOB_INFO_1 * buffer = get_job_info(printer, job_id);
	if(buffer == NULL) return NULL;
	
	jstring return_me = (*env)->NewStringUTF(env, buffer[0].pDocument);
	free(buffer);
	return return_me;
}

JNIEXPORT jint JNICALL Java_jwinspool_JWinSpool_getJobStatus(JNIEnv * env, jclass class, jint printer, jint job_id)
{
	JOB_INFO_1 * buffer = get_job_info(printer, job_id);
	if(buffer == NULL) return 0;
	
	jint return_me = buffer[0].Status;
	free(buffer);
	return return_me;
}

JNIEXPORT jboolean JNICALL Java_jwinspool_JWinSpool_setJob(JNIEnv * env, jclass class, jint printer, jint job_id, jint command)
{
	return SetJob((HANDLE)printer, job_id, 0, NULL, command);
}

static PRINTER_INFO_2 * get_printer_info(jint printer)
{
	PRINTER_INFO_2 * buffer = NULL;
	DWORD length;
	
	GetPrinter((HANDLE)printer, 2, NULL, 0, &length);
	if(length == 0) return NULL;
	buffer = (PRINTER_INFO_2 *)malloc(length);

	if(!GetPrinter((HANDLE)printer, 2, (char *)buffer, length, &length)) return NULL;
	return buffer;
}

JNIEXPORT jint JNICALL Java_jwinspool_JWinSpool_getPrinterStatus(JNIEnv * env, jclass class, jint printer)
{
	PRINTER_INFO_2 * buffer = get_printer_info(printer);
	if(buffer == NULL) return 0;

	jint return_me = buffer->Status;
	free(buffer);
	return return_me;
}

JNIEXPORT jint JNICALL Java_jwinspool_JWinSpool_getNumJobs(JNIEnv * env, jclass class, jint printer)
{
	PRINTER_INFO_2 * buffer = get_printer_info(printer);
	if(buffer == NULL) return 0;

	jint return_me = buffer->cJobs;
	free(buffer);
	return return_me;
}
