package jhwnd;

/**
 * A class to wrap around the native Windows message handling.
 *
 * @author Greg Hackmann
 */
public abstract class JHWnd
{
	public static final int WM_KEYDOWN = 256;
	public static final int WM_KEYUP = 257;
	public static final int WM_CHAR = 258;
	public static final int WM_COMMAND = 273;
	public static final int WM_USER = 1024;
	
	/**
	 * Get a handle to the window with the given class.
	 *
	 * @param title the class of the window to find
	 * @return the handle of the window, or 0 if the window was not found
	 */
	public static native int findWindow(String title);
	
	/**
	 * Send a message to the specified window.  This method blocks until the
	 * window acknowledges that it received the message.
	 *
	 * @param hWnd the handle of the window to send the message to
	 * @param type the type of message to send (one of the WM_* constants above,
	 * or any other WM_* constant defined in winuser.h)
	 * @param wParam the first parameter of the message (its meaning varies
	 * based on the type of message sent)
	 * @param lParam the second parameter of the message (its meaning varies
	 * based on the type of message sent)
	 * @return the message's return value
	 */
	public static native long sendMessage(int hWnd, int type, int wParam, int lParam);
	
	/**
	 * Post a message to the specified window.  This method is a non-blocking
	 * version of {@link #sendMessage(int, int, int, int)}.
	 *
	 * @param hWnd the handle of the window to post the message to
	 * @param type the type of message to post (one of the WM_* constants above,
	 * or any other WM_* constant defined in winuser.h)
	 * @param wParam the first parameter of the message (its meaning varies
	 * based on the type of message posted)
	 * @param lParam the second parameter of the message (its meaning varies
	 * based on the type of message posted)
	 * @return true if the message was successfully posted, false if it failed
	 */
	public static native boolean postMessage(int hWnd, int type, int wParam, int lParam);
	
	/**
	 * Get the title of the specified window.
	 *
	 * @param hWnd the handle of the window to read the title from
	 * @return the window's title
	 */
	public static native String getWindowText(int hWnd);
}
