package jwia;

/**
 * A class to wrap around the native Windows Image Acquisition API.
 *
 * @author Greg Hackmann
 */
public abstract class JWIA
{
	/**
	 * Open the Windows Image Acqusition device manager.
	 *
	 * @return a handle to the device manager, or 0 if the device
	 * manager could not be opened
	 */
	public static native int openManager();
	
	/**
	 * Close the provided WIA device manager.
	 *
	 * @param manager the device manager to close
	 */
	public static native void closeManager(int manager);
	
	/**
	 * Open the first camera device that has streaming video.
	 *
	 * @param manager the WIA device manager (see {@link #openManager()})
	 */
	public static native int openCamera(int manager);

	/**
	 * Close the provided camera device.
	 *
	 * @param camera the camera to close
	 */
	public static native void closeCamera(int camera);

	/**
	 * Tell the provided camera to take a snapshot.
	 *
	 * @param camera the camera to use
	 * @return the filename of the picture taken, or null if the snapshot failed
	 */
	public static native String takeSnapshot(int camera);
}
