package jwinspool;

/**
 * A class to wrap around the native Windows print spooler.
 *
 * @author Greg Hackmann
 */
public abstract class JWinSpool
{
	public static final int JOB_CONTROL_PAUSE = 1;
	public static final int JOB_CONTROL_RESUME = 2;
	public static final int JOB_CONTROL_CANCEL = 3;
	public static final int JOB_CONTROL_RESTART = 4;
	public static final int JOB_CONTROL_DELETE = 5;
	
	public static final int JOB_STATUS_PAUSED = 1;
	public static final int JOB_STATUS_ERROR = 2;
	public static final int JOB_STATUS_DELETING = 4;
	public static final int JOB_STATUS_SPOOLING = 8;
	public static final int JOB_STATUS_PRINTING = 16;
	public static final int JOB_STATUS_OFFLINE = 32;
	public static final int JOB_STATUS_PAPEROUT = 0x40;
	public static final int JOB_STATUS_PRINTED = 0x80;
	public static final int JOB_STATUS_DELETED = 0x100;
	public static final int JOB_STATUS_BLOCKED_DEVQ	= 0x200;
	public static final int JOB_STATUS_USER_INTERVENTION = 0x400;
	
	public static final int PRINTER_STATUS_PAUSED = 1;
	public static final int PRINTER_STATUS_ERROR = 2;
	public static final int PRINTER_STATUS_PENDING_DELETION = 4;
	public static final int PRINTER_STATUS_PAPER_JAM = 8;
	public static final int PRINTER_STATUS_PAPER_OUT = 0x10;
	public static final int PRINTER_STATUS_MANUAL_FEED = 0x20;
	public static final int PRINTER_STATUS_PAPER_PROBLEM = 0x40;
	public static final int PRINTER_STATUS_OFFLINE = 0x80;
	public static final int PRINTER_STATUS_IO_ACTIVE = 0x100;
	public static final int PRINTER_STATUS_BUSY = 0x200;
	public static final int PRINTER_STATUS_PRINTING = 0x400;
	public static final int PRINTER_STATUS_OUTPUT_BIN_FULL = 0x800;
	public static final int PRINTER_STATUS_NOT_AVAILABLE = 0x1000;
	public static final int PRINTER_STATUS_WAITING = 0x2000;
	public static final int PRINTER_STATUS_PROCESSING = 0x4000;
	public static final int PRINTER_STATUS_INITIALIZING = 0x8000;
	public static final int PRINTER_STATUS_WARMING_UP = 0x10000;
	public static final int PRINTER_STATUS_TONER_LOW = 0x20000;
	public static final int PRINTER_STATUS_NO_TONER = 0x40000;
	public static final int PRINTER_STATUS_PAGE_PUNT = 0x80000;
	public static final int PRINTER_STATUS_USER_INTERVENTION = 0x100000;
	public static final int PRINTER_STATUS_OUT_OF_MEMORY = 0x200000;
	public static final int PRINTER_STATUS_DOOR_OPEN = 0x400000;
	public static final int PRINTER_STATUS_SERVER_UNKNOWN = 0x800000;
	public static final int PRINTER_STATUS_POWER_SAVE = 0x1000000;

	/**
	 * Get the name of the default printer.
	 *
	 * @return the default printer's name
	 */
	public static native String getDefaultPrinter();
	
	
	/**
	 * Open and get a handle to the printer with the specified name.
	 *
	 * @param name the printer's name
	 * @return the handle to the printer, or 0 if the printer could not be
	 * opened
	 */
	public static native int openPrinter(String name);
	
	/**
	 * Close the specified printer.
	 *
	 * @param printer the printer's handle
	 * @return true if the printer was closed, or false if it failed
	 */
	public static native boolean closePrinter(int printer);

	/**
	 * Get the ID of a printer's job.
	 *
	 * @param printer the printer's handle
	 * @param jobNum the index of the job, where 0 is the printer's first job
	 * @return the ID of the job, or 0 if the job's ID could not be read
	 */
	public static native int getJobID(int printer, int jobNum);
	
	/**
	 * Get the name of a printer's job.
	 *
	 * @param printer the printer's handle
	 * @param jobID the job's ID
	 * @return the name of the job, or null if the job's name could not be read
	 */
	public static native String getJobName(int printer, int jobID);
	
	/**
	 * Get a job's status.  This is specified bitwise using the JOB_STATUS_*
	 * constants defined above.
	 *
	 * @param printer the printer's handle
	 * @param jobID the job's ID
	 * @return the status of the job, or 0 if the job's status could not be read
	 */
	public static native int getJobStatus(int printer, int jobID);
	
	/**
	 * Set a job's status.  This is specified bitwise using the JOB_STATUS_*
	 * constants defined above.
	 *
	 * @param printer the printer's handle
	 * @param jobID the job's ID
	 * @param command the status of the job
	 * @return true if the status changed succeeded, false if it failed
	 */
	public static native boolean setJob(int printer, int jobID, int command);

	/**
	 * Get the status of the specified printer.  This is specified bitwise using
	 * the PRINTER_STATUS_* constants defined above.
	 *
	 * @param printer the printer's handle
	 * @return the status of the printer, or 0 if the printer's status could
	 * not be read
	 */
	public static native int getPrinterStatus(int printer);
	
	/**
	 * Get the number of jobs in the specified printer's queue.
	 *
	 * @param printer the printer's handle
	 * @return the number of jobs in the printer's queue, or -1 if the printer's
	 * queue could not be read
	 */
	public static native int getNumJobs(int printer);
}
