package uremoteagents;

import java.net.InetAddress;
import java.awt.event.*;
import java.awt.event.WindowAdapter;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.swing.JFrame;
import javax.swing.JLabel;

import com.sun.image.codec.jpeg.*;
import jwia.JWIA;
import uremote.*;

public class CameraAgent implements DeviceDefinition
{
	static
	{
		System.loadLibrary("JWIA");
	}
	
	private int manager, camera;
	private String ip;
	private BufferedImage previousImage, image;
	private GenericDeviceAgent agent;
	private Timer timer;
	private TimerTask task;
	
	public CameraAgent()
	{
		manager = JWIA.openManager();
		camera = JWIA.openCamera(manager);
		previousImage = image = null;
		
		timer = new Timer(false);
		task = new MotionTask();
		//timer.scheduleAtFixedRate(task, 0, 2000);
	
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("CameraAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				task.cancel();
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	class MotionTask extends TimerTask
	{
		public void run()
		{
			if(agent == null) return;
			takePicture();
			if(motionDetected(.1, .01)) agent.changeState("Picture", getState("Advertisement"));
		}
	}
	
	private boolean motionDetected(double threshold, double percentage)
	{
		if(previousImage == null || image == null) return false;
		if(previousImage.getWidth() != image.getWidth() ||
		   previousImage.getHeight() != image.getHeight()) return false;
		
		int prevRGB [] = previousImage.getRGB(0, 0, previousImage.getWidth(), previousImage.getHeight(), null, 0, previousImage.getWidth());
		int currentRGB [] = image.getRGB(0, 0, image.getWidth(), image.getHeight(), null, 0, image.getWidth());
		
		double pixelsChanged = 0;
		for(int i = 0; i < currentRGB.length; i++)
		{
			byte prevR = (byte)((prevRGB[i] & 0xFF0000) >> 16),
				 currentR = (byte)((currentRGB[i] & 0xFF0000) >> 16),
			     prevG = (byte)((prevRGB[i] & 0xFF00) >> 8),
				 currentG = (byte)((currentRGB[i] & 0xFF00) >> 8),
			     prevB = (byte)(prevRGB[i] & 0xFF),
				 currentB = (byte)(currentRGB[i] & 0xFF);
			
			byte deltaR = (byte)Math.abs(prevR - currentR),
				 deltaG = (byte)Math.abs(prevG - currentG),
	 			 deltaB = (byte)Math.abs(prevB - currentB);
			double diff = (deltaR + deltaG + deltaB) / 3.0 / 256.0;

			if(diff > threshold) pixelsChanged++;
		}
		
//		System.out.println(pixelsChanged + " of " + currentRGB.length + " pixels changed");
		return ((pixelsChanged / currentRGB.length) > percentage);
	}
	
	private void takePicture()
	{
		final String filename = JWIA.takeSnapshot(camera);
		if(filename == null) return;
		previousImage = image;

		try
		{
			FileInputStream in = new FileInputStream(filename);
			JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
			image = decoder.decodeAsBufferedImage();
			timer.schedule(new TimerTask()
			{
				public void run()
				{
					File deleteMe = new File(filename);
					if(!deleteMe.delete()) deleteMe.deleteOnExit();
				}
			}, 5000);
		}
		catch(Exception e)
		{
			image = null;
			e.printStackTrace();
		}
	}

	
	public String getName()
	{
		return "Camera";
	}

	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "X10";
	}
	
	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		takePicture();
		agent.changeState("Picture", getState("Advertisement"));
	}
	
	public void reactToSlider(String dummy, int dummy2)
	{
	}

	public String[] getFunctions()
	{
		return new String [] { "Snap", "Picture" };
	}

	public String getHelpText(String functionName)
	{
		return "Take a picture";
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		if(functionName.equals("Snap")) return new ButtonAdvertisement("Snap", false, "RECORD", 0, 0, 1, 3);
		else return new ImageAdvertisement("Picture", 1, 0, 4, 3);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Snap")) return new ButtonState(true, false);
		else
		{
			BufferedImage img = image;;
			if(img == null) img = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
			double xScale = 192.0 / img.getWidth(), yScale = 144.0 / img.getHeight();
			AffineTransform scaleOp = AffineTransform.getScaleInstance(xScale, yScale);
			AffineTransformOp scaler = new AffineTransformOp(scaleOp, AffineTransformOp.TYPE_BILINEAR);
			BufferedImage scaled = scaler.filter(img, null);
			return new ImageState(scaled, "Picture");
		}
	}
	
	public void shutDown()
	{
		task.cancel();
		/*	if(camera != 0) JWIA.closeCamera(camera);
		 if(manager != 0) JWIA.closeManager(manager);*/
	}

}
