package uremoteagents;

import java.net.InetAddress;
import java.util.*;
import java.awt.event.*;
import javax.swing.JLabel;
import javax.swing.JFrame;
import com.apple.cocoa.foundation.*;
import uremote.*;

public class DVDPlayerAgent implements DeviceDefinition
{
	private GenericDeviceAgent agent;
	private String ip;
	private Hashtable adTable, helpTable;
	
	private String functions [] = { "VolumeDown", "Mute", "VolumeUp", "Previous",
									 "Next", "Rewind", "FastForward", "Play", "Pause",
									 "Stop", "Up", "Left", "Right", "Down", "Enter" };
		
	public DVDPlayerAgent()
	{
		com.apple.cocoa.application.NSApplication.sharedApplication();
		
		adTable = new Hashtable();
		
		adTable.put("VolumeUp", new ButtonAdvertisement("VolumeUp", false, "ARROWUP", 3, 0));
		adTable.put("Mute", new ButtonAdvertisement("Mute", false, "MUTE", 2, 0));
		adTable.put("VolumeDown", new ButtonAdvertisement("VolumeDown", false, "ARROWDOWN", 1, 0));
		
		adTable.put("Previous", new ButtonAdvertisement("Previous", false, "PREVIOUS", 0, 1));
		adTable.put("Rewind", new ButtonAdvertisement("Rewind", false, "REWIND", 1, 1));
		adTable.put("FastForward", new ButtonAdvertisement("FastForward", false, "FASTFORWARD", 3, 1));
		adTable.put("Next", new ButtonAdvertisement("Next", false, "NEXT", 4, 1));
		
		adTable.put("Play", new ButtonAdvertisement("Play", false, "PLAY", 0, 2, 2, 1));
		adTable.put("Pause", new ButtonAdvertisement("Pause", false, "PAUSE", 2, 2));
		adTable.put("Stop", new ButtonAdvertisement("Stop", false, "STOP", 3, 2, 2, 1));
		
		adTable.put("Up", new ButtonAdvertisement("Up", false, "ARROWUP", 2, 4));
		adTable.put("Left", new ButtonAdvertisement("Left", false, "ARROWLEFT", 1, 5));
		adTable.put("Enter", new ButtonAdvertisement("Enter", false, "ARROWENTER", 2, 5));
		adTable.put("Right", new ButtonAdvertisement("Right", false, "ARROWRIGHT", 3, 5));
		adTable.put("Down", new ButtonAdvertisement("Down", false, "ARROWDOWN", 2, 6));
		
		helpTable = new Hashtable();
		
		helpTable.put("VolumeUp", "Increase the volume");
		helpTable.put("Mute", "Mute or unmute the movie");
		helpTable.put("VolumeDown", "Decrease the volume");
		helpTable.put("Previous", "Go to the previous chapter");
		helpTable.put("Rewind", "Rewind");
		helpTable.put("FastForward", "Fast-forward");
		helpTable.put("Next", "Go to the next chapter");
		helpTable.put("Play", "Play the movie");
		helpTable.put("Pause", "Pause or unpause the movie");
		helpTable.put("Stop", "Stop the movie");
		helpTable.put("Up", "Menu Up");
		helpTable.put("Down", "Menu Down");
		helpTable.put("Left", "Menu Left");
		helpTable.put("Right", "Menu Right");
		helpTable.put("Enter", "Menu Enter");
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("DVDPlayerAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	private NSAppleEventDescriptor script(String text)
	{
		text = "tell application \"DVD Player\"\n" + text + "\nend tell";
		return new NSAppleScript(text).execute(new NSMutableDictionary());
	}
	
	public void mute() { script("set audio muted to not audio muted"); }
	public void volumeUp() { script("set audio volume to audio volume + 16"); }
	public void volumeDown() { script("set audio volume to audio volume - 16"); }
	
	public void previousChapter() { script("set chapter to chapter - 1"); }
	public void nextChapter() { script("set chapter to chapter + 1"); }
	
	public void play() { script("play dvd"); }
	public void pause() { script("pause dvd"); }
	public void stop() { script("stop dvd"); }
	
	public void fastForward() { script("fast forward dvd"); }
	public void rewind() { script("rewind dvd"); }
	
	public void menuUp() { script("press up arrow key"); }
	public void menuLeft() { script("press left arrow key"); }
	public void menuRight() { script("press right arrow key"); }
	public void menuDown() { script("press down arrow key"); }
	public void menuEnter() { script("press enter key"); }
	
	public String getName()
	{
		return "DVD Player";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "DVD";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Play")) play();
		else if(functionName.equals("Stop")) stop();
		else if(functionName.equals("Pause")) pause();
		else if(functionName.equals("Previous")) previousChapter();
		else if(functionName.equals("Next")) nextChapter();
		else if(functionName.equals("Rewind")) rewind();
		else if(functionName.equals("FastForward")) fastForward();
		else if(functionName.equals("VolumeDown")) volumeDown();
		else if(functionName.equals("VolumeUp")) volumeUp();
		else if(functionName.equals("Mute")) mute();
		else if(functionName.equals("Up")) menuUp();
		else if(functionName.equals("Down")) menuDown();
		else if(functionName.equals("Left")) menuLeft();
		else if(functionName.equals("Right")) menuRight();
		else if(functionName.equals("Enter")) menuEnter();
	}

	public void reactToSlider(String functionName, int value)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		return new ButtonState(true, false);
	}

	public void shutDown()
	{
	}
}
