package uremoteagents;

import java.net.InetAddress;
import java.util.Hashtable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import uremote.*;

public class MouseAgent implements DeviceDefinition
{
	private String ip;
	private GenericDeviceAgent agent;
	private Dimension screenSize;
	private Robot bot;
	private int x, y;
	
	private String functions [] = { "Up", "Down", "Left", "Right", "Button1", "Button2", "Button3" };
	private Hashtable adTable, helpTable;
	
	public MouseAgent()
	{
		try
		{
			bot = new Robot();
		}
		catch(AWTException e)
		{
			e.printStackTrace();
		}
		
		adTable = new Hashtable();
		adTable.put("Up", new ButtonAdvertisement("Up", false, "ARROWUP", 2, 1));
		adTable.put("Down", new ButtonAdvertisement("Down", false, "ARROWDOWN", 2, 3));
		adTable.put("Left", new ButtonAdvertisement("Left", false, "ARROWLEFT", 1, 2));
		adTable.put("Right", new ButtonAdvertisement("Right", false, "ARROWRIGHT", 3, 2));
		adTable.put("Button1", new ButtonAdvertisement("Button1", false, "RECORD", 0, 0, 2, 1));
		adTable.put("Button2", new ButtonAdvertisement("Button2", false, "RECORD", 2, 0, 1, 1));
		adTable.put("Button3", new ButtonAdvertisement("Button3", false, "RECORD", 3, 0, 2, 1));

		helpTable = new Hashtable();
		helpTable.put("Up", "Move the mouse up");
		helpTable.put("Down", "Move the mouse down");
		helpTable.put("Left", "Move the mouse left");
		helpTable.put("Right", "Move the mouse right");
		helpTable.put("Button1", "Click the left mouse button");
		helpTable.put("Button2", "Click the middle mouse button");
		helpTable.put("Button3", "Click the right mouse button");
		
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		x = screenSize.width / 2;
		y = screenSize.height / 2;
		bot.mouseMove(x, y);
		/* Start the mouse in the middle of the screen */
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("MouseAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}

	private void moveMouse(int deltaX, int deltaY)
	{
		x += deltaX; y += deltaY;
		if(x < 0) x = 0;
		if(y < 0) y = 0;
		if(x >= screenSize.width) x = screenSize.width - 1;
		if(y >= screenSize.height) y = screenSize.height - 1;
		/* Figure out where to put the mouse, making sure it doesn't fly off the
		   side of the screen */
		
		bot.mouseMove(x, y);
		/* Now actually move it */
	}
	
	public String getName()
	{
		return "PC";
	}

	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "COMPUTER";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Up")) moveMouse(0, -30);
		else if(functionName.equals("Down")) moveMouse(0, 30);
		else if(functionName.equals("Left")) moveMouse(-30, 0);
		else if(functionName.equals("Right")) moveMouse(30, 0);
		else if(functionName.equals("Button1"))
		{
			bot.mousePress(InputEvent.BUTTON1_MASK);
			bot.mouseRelease(InputEvent.BUTTON1_MASK);
		}
		else if(functionName.equals("Button2"))
		{
			bot.mousePress(InputEvent.BUTTON2_MASK);
			bot.mouseRelease(InputEvent.BUTTON2_MASK);
		}
		else if(functionName.equals("Button3"))
		{
			bot.mousePress(InputEvent.BUTTON3_MASK);
			bot.mouseRelease(InputEvent.BUTTON3_MASK);
		}
	}
	
	public void reactToSlider(String dummy, int dummy2)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		return new ButtonState(true, false);
	}
	
	public void shutDown()
	{
	}
}
