package uremoteagents;

import jhwnd.*;
import uremote.*;

import java.awt.event.*;
import java.net.InetAddress;
import java.util.*;

import javax.swing.*;

public class PowerDVDAgent implements DeviceDefinition
{
	private int hWnd = 0;
	
	private final static int VK_RETURN = 13;
	private final static int VK_LEFT = 37;
	private final static int VK_UP = 38;
	private final static int VK_RIGHT = 39;
	private final static int VK_DOWN = 40;
	private final static int VK_ADD = 0x6B;
	private final static int VK_SUBTRACT = 0x6D;
	
	private GenericDeviceAgent agent;
	private String ip;
	private Hashtable adTable, helpTable;
	
	private String functions [] = { "VolumeDown", "Mute", "VolumeUp", "Previous",
									 "Next", "Rewind", "FastForward", "Play", "Pause",
									 "Stop", "Up", "Left", "Right", "Down", "Enter" };
	
	static
	{
		System.loadLibrary("JHWnd");
	}
		
	public PowerDVDAgent()
	{
		hWnd = JHWnd.findWindow("CyberLink Video Window Class");
		if(hWnd == 0)
		{
			JOptionPane.showMessageDialog(null, "PowerDVD is not running.", "Could not load agent", JOptionPane.ERROR_MESSAGE);
			return;
		}
		adTable = new Hashtable();
		
		adTable.put("VolumeUp", new ButtonAdvertisement("VolumeUp", false, "ARROWUP", 3, 0));
		adTable.put("Mute", new ButtonAdvertisement("Mute", false, "MUTE", 2, 0));
		adTable.put("VolumeDown", new ButtonAdvertisement("VolumeDown", false, "ARROWDOWN", 1, 0));
		
		adTable.put("Previous", new ButtonAdvertisement("Previous", false, "PREVIOUS", 0, 1));
		adTable.put("Rewind", new ButtonAdvertisement("Rewind", false, "REWIND", 1, 1));
		adTable.put("FastForward", new ButtonAdvertisement("FastForward", false, "FASTFORWARD", 3, 1));
		adTable.put("Next", new ButtonAdvertisement("Next", false, "NEXT", 4, 1));
		
		adTable.put("Play", new ButtonAdvertisement("Play", false, "PLAY", 0, 2, 2, 1));
		adTable.put("Pause", new ButtonAdvertisement("Pause", false, "PAUSE", 2, 2));
		adTable.put("Stop", new ButtonAdvertisement("Stop", false, "STOP", 3, 2, 2, 1));
		
		adTable.put("Up", new ButtonAdvertisement("Up", false, "ARROWUP", 2, 4));
		adTable.put("Left", new ButtonAdvertisement("Left", false, "ARROWLEFT", 1, 5));
		adTable.put("Enter", new ButtonAdvertisement("Enter", false, "ARROWENTER", 2, 5));
		adTable.put("Right", new ButtonAdvertisement("Right", false, "ARROWRIGHT", 3, 5));
		adTable.put("Down", new ButtonAdvertisement("Down", false, "ARROWDOWN", 2, 6));
		
		helpTable = new Hashtable();
		
		helpTable.put("VolumeUp", "Increase the volume");
		helpTable.put("Mute", "Mute or unmute the movie");
		helpTable.put("VolumeDown", "Decrease the volume");
		helpTable.put("Previous", "Go to the previous chapter");
		helpTable.put("Rewind", "Rewind");
		helpTable.put("FastForward", "Fast-forward");
		helpTable.put("Next", "Go to the next chapter");
		helpTable.put("Play", "Play the movie");
		helpTable.put("Pause", "Pause or unpause the movie");
		helpTable.put("Stop", "Stop the movie");
		helpTable.put("Up", "Menu Up");
		helpTable.put("Down", "Menu Down");
		helpTable.put("Left", "Menu Left");
		helpTable.put("Right", "Menu Right");
		helpTable.put("Enter", "Menu Enter");
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("PowerDVDAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	private boolean postKey(int cmd)
	{
		return JHWnd.postMessage(hWnd, JHWnd.WM_KEYDOWN, cmd, 0);
	}
	
	public void mute() { postKey('Q'); }
	public void volumeUp() { postKey(VK_ADD); }
	public void volumeDown() { postKey(VK_SUBTRACT); }
	
	public void previousChapter() { postKey('P'); }
	public void nextChapter() { postKey('N'); }
	
	public void play() { menuEnter(); }
	public void pause() { postKey(' '); }
	public void stop() { postKey('S'); }
	
	public void fastForward() { postKey('F'); }
	public void rewind() { postKey('B'); }
	
	public void menuUp() { postKey(VK_UP); }
	public void menuLeft() { postKey(VK_LEFT); }
	public void menuRight() { postKey(VK_RIGHT); }
	public void menuDown() { postKey(VK_DOWN); }
	public void menuEnter() { postKey(VK_RETURN); }
	
	public String getName()
	{
		return "PowerDVD";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "DVD";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Play")) play();
		else if(functionName.equals("Stop")) stop();
		else if(functionName.equals("Pause")) pause();
		else if(functionName.equals("Previous")) previousChapter();
		else if(functionName.equals("Next")) nextChapter();
		else if(functionName.equals("Rewind")) rewind();
		else if(functionName.equals("FastForward")) fastForward();
		else if(functionName.equals("VolumeDown")) volumeDown();
		else if(functionName.equals("VolumeUp")) volumeUp();
		else if(functionName.equals("Mute")) mute();
		else if(functionName.equals("Up")) menuUp();
		else if(functionName.equals("Down")) menuDown();
		else if(functionName.equals("Left")) menuLeft();
		else if(functionName.equals("Right")) menuRight();
		else if(functionName.equals("Enter")) menuEnter();
	}

	public void reactToSlider(String functionName, int value)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		return new ButtonState(true, false);
	}

	public void shutDown()
	{
	}
}
