package uremoteagents;

import jwinspool.JWinSpool;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.print.*;
import javax.swing.JLabel;
import javax.swing.JFrame;

import java.net.InetAddress;
import java.util.*;
import uremote.*;

public class PrinterAgent implements DeviceDefinition
{
	static
	{
		System.loadLibrary("JWinSpool");
	}
	
	private GenericDeviceAgent agent;
	private Hashtable adTable, helpTable;
	private String [] functions = { "PrevPrinter", "Printer", "NextPrinter",
	                                "PrevJob", "Job", "NextJob",
	                                "Pause", "Stop", "Test" };
	private int jobIndex = 0, printerIndex = 0;
	private String ip;
	private RefreshTask task;
	
	public PrinterAgent()
	{
		printerIndex = getDefaultPrinter();
		adTable = new Hashtable();
		
		adTable.put("PrevPrinter", new ButtonAdvertisement("PrevPrinter", false, "REWIND", 0, 0));
		adTable.put("Printer", new LabelAdvertisement("Printer", 1, 0, 3, 1));
		adTable.put("NextPrinter", new ButtonAdvertisement("NextPrinter", false, "FASTFORWARD", 4, 0));
		adTable.put("PrevJob", new ButtonAdvertisement("PrevJob", false, "PREVIOUS", 0, 1));
		adTable.put("Job", new LabelAdvertisement("Job", 1, 1, 3, 1));
		adTable.put("NextJob", new ButtonAdvertisement("NextJob", false, "NEXT", 4, 1));
		adTable.put("Pause", new ButtonAdvertisement("Pause", false, "PAUSE", 1, 2));
		adTable.put("Stop", new ButtonAdvertisement("Stop", false, "STOP", 2, 2));
		adTable.put("Test", new ButtonAdvertisement("Test", false, "RECORD", 3, 2));
										
		helpTable = new Hashtable();
		
		helpTable.put("PrevPrinter", "Show the previous printer");
		helpTable.put("Printer", "");
		helpTable.put("NextPrinter", "Show the next printer");
		helpTable.put("PrevJob", "Show the previous printer job");
		helpTable.put("Job", "");
		helpTable.put("NextJob", "Show the next printer job");
		helpTable.put("Pause", "Pause the current print job");
		helpTable.put("Stop", "Cancel the current print job");
		helpTable.put("Test", "Print a test page");
		
		Timer refresh = new Timer(false);
		task = new RefreshTask();
		refresh.scheduleAtFixedRate(task, 0, 5000);
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("PrinterAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	class RefreshTask extends TimerTask
	{
		public void run()
		{
			if(printerEmpty(printerIndex)) jobIndex = 0;
			else if(jobIndex >= getNumJobs(getPrinterName(printerIndex))) jobIndex = getNumJobs(getPrinterName(printerIndex));
			
			String functions [] = new String []
			{
				"Printer",
				"Job",
				"PrevPrinter",
				"NextPrinter",
				"PrevJob",
				"NextJob",
				"Pause",
				"Stop"
			};
			ComponentState states [] = new ComponentState[functions.length];
			
			for(int i = 0; i < functions.length; i++) states[i] = getState(functions[i]);
			agent.changeState(functions, states);
		}
	}
	
	public String getDefaultPrinterName()
	{
		return JWinSpool.getDefaultPrinter();
	}
	
	public int getDefaultPrinter()
	{
		String name = getDefaultPrinterName();
		PrintService [] searchMe = PrinterJob.lookupPrintServices();
		
		for(int i = 0; i < searchMe.length; i++)
		{
			if(searchMe[i].getName().equals(name)) return i;
		}
		return -1;
	}
	
	public int getNumPrinters()
	{
		return PrinterJob.lookupPrintServices().length;
	}
	
	public String getPrinterName(int index)
	{
		return PrinterJob.lookupPrintServices()[index].getName();
	}
	
	public String getJobName(String printerName, int index)
	{
		int printerHandle = JWinSpool.openPrinter(printerName);
		if(printerHandle == 0) return "";
		
		int jobID = JWinSpool.getJobID(printerHandle, index);
		if(jobID == 0)
		{
			JWinSpool.closePrinter(printerHandle);
			return "";
		}
		
		String jobName = JWinSpool.getJobName(printerHandle, jobID);
		JWinSpool.closePrinter(printerHandle);
		return (jobName == null ? "" : jobName);
	}
	
	public boolean addJob(String printerName, Printable printMe, String jobName)
	{
		PrinterJob job = PrinterJob.getPrinterJob();
		PrintService printer = getPrinterService(printerName);
		if(printer == null) return false;
		
		try
		{
			job.setPrintService(printer);
	
			Book book = new Book();
			book.append(printMe, new PageFormat());
			job.setPageable(book);
			job.setJobName(jobName);
	
			job.print();
			return true;
		}
		catch(PrinterException e)
		{
			return false;
		}
	}
	
	public boolean deleteJob(String printerName, int index)
	{
		return setJobStatus(printerName, index, JWinSpool.JOB_CONTROL_DELETE);
	}
	
	public boolean pauseJob(String printerName, int index)
	{
		return setJobStatus(printerName, index, JWinSpool.JOB_CONTROL_PAUSE);
	}
	
	public boolean unpauseJob(String printerName, int index)
	{
		return setJobStatus(printerName, index, JWinSpool.JOB_CONTROL_RESUME);
	}
	
	public boolean isJobPaused(String printerName, int index)
	{
		return (getJobStatus(printerName, index) & JWinSpool.JOB_STATUS_PAUSED) != 0;
	}
	
	private int getJobStatus(String printerName, int index)
	{
		int printerHandle = JWinSpool.openPrinter(printerName);
		if(printerHandle == 0) return 0;
		
		int jobID = JWinSpool.getJobID(printerHandle, index);
		if(jobID == 0)
		{
			JWinSpool.closePrinter(printerHandle);
			return 0;
		}
		
		int returnMe = JWinSpool.getJobStatus(printerHandle, jobID);
		JWinSpool.closePrinter(printerHandle);
		return returnMe;
	}
	
	private boolean setJobStatus(String printerName, int index, int command)
	{
		int printerHandle = JWinSpool.openPrinter(printerName);
		if(printerHandle == 0) return false;
		
		int jobID = JWinSpool.getJobID(printerHandle, index);
		if(jobID == 0)
		{
			JWinSpool.closePrinter(printerHandle);
			return false;
		}
		
		boolean returnMe = JWinSpool.setJob(printerHandle, jobID, command);
		JWinSpool.closePrinter(printerHandle);
		return returnMe;
	}
	
	public boolean isPrinterAvailable(String printerName)
	{
		return (getPrinterStatus(printerName) & JWinSpool.PRINTER_STATUS_NOT_AVAILABLE) == 0;
	}

	public boolean isPrinterBusy(String printerName)
	{
		return (getPrinterStatus(printerName) & JWinSpool.PRINTER_STATUS_BUSY) != 0;
	}
	
	public boolean isPrinterError(String printerName)
	{
		return (getPrinterStatus(printerName) & JWinSpool.PRINTER_STATUS_ERROR) != 0;
	}
		
	public boolean isPrinterPaused(String printerName)
	{
		return (getPrinterStatus(printerName) & JWinSpool.PRINTER_STATUS_PAUSED) != 0;
	}
	
	public int getNumJobs(String printerName)
	{
		int printerHandle = JWinSpool.openPrinter(printerName);
		if(printerHandle == 0) return 0;
		
		return JWinSpool.getNumJobs(printerHandle);
	}

	private int getPrinterStatus(String printerName)
	{
		int printerHandle = JWinSpool.openPrinter(printerName);
		if(printerHandle == 0) return 0;
		
		int returnMe = JWinSpool.getPrinterStatus(printerHandle);
		JWinSpool.closePrinter(printerHandle);
		return returnMe;
	}
	
	private PrintService getPrinterService(String printerName)
	{
		PrintService [] searchMe = PrinterJob.lookupPrintServices();
		for (int i = 0; i < searchMe.length; i++)
		{
			if(searchMe[i].getName().equals(printerName)) return searchMe[i];
		}
		
		return null;
	}
	
	public String getName()
	{
		return "Printer";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "PRINTER";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("PrevPrinter"))
		{
			if(printerIndex > 0) printerIndex--;
		}
		else if(functionName.equals("NextPrinter"))
		{
			if(printerIndex < getNumPrinters() - 1) printerIndex++;
		}
		else if(functionName.equals("PrevJob"))
		{
			if(jobIndex > 0) jobIndex--;
		}
		else if(functionName.equals("NextJob"))
		{
			if(jobIndex < getNumJobs(getPrinterName(printerIndex))) jobIndex++;
		}
		else if(functionName.equals("Pause"))
		{
			if(isJobPaused(getPrinterName(printerIndex), jobIndex)) unpauseJob(getPrinterName(printerIndex), jobIndex);
			else pauseJob(getPrinterName(printerIndex), jobIndex);
		}
		else if(functionName.equals("Stop"))
		{
			deleteJob(getPrinterName(printerIndex), jobIndex);
		}
		else if(functionName.equals("Test"))
		{
			addJob(getPrinterName(printerIndex), new TestPage(), "PrinterAgent Test Page");
		}
		
		task.run();
	}

	public void reactToSlider(String functionName, int value)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Printer"))
		{
			return new LabelState(true, getPrinterName(printerIndex));
		}
		else if(functionName.equals("Job"))
		{
			if(printerEmpty(printerIndex)) return new LabelState(false, "(Printer queue is empty)");
			return new LabelState(true, getJobName(getPrinterName(printerIndex), jobIndex));
		}
		else if(functionName.equals("PrevPrinter"))
		{
			return new ButtonState(printerIndex > 0, false);
		}
		else if(functionName.equals("NextPrinter"))
		{
			return new ButtonState(printerIndex < getNumPrinters() - 1, false);
		}
		else if(functionName.equals("PrevJob"))
		{
			return new ButtonState(!printerEmpty(printerIndex) && jobIndex > 0, false);
		}
		else if(functionName.equals("NextJob"))
		{
			return new ButtonState(!printerEmpty(printerIndex) && jobIndex < getNumJobs(getPrinterName(printerIndex)) - 1, false);
		}
		else if(functionName.equals("Pause"))
		{
			return new ButtonState(!printerEmpty(printerIndex), isJobPaused(getPrinterName(printerIndex), jobIndex));
		}
		else if(functionName.equals("Stop"))
		{
			return new ButtonState(!printerEmpty(printerIndex), false);
		}
		else return new ButtonState(true, false);
	}
	
	private boolean printerEmpty(int printerIndex)
	{
		return getNumJobs(getPrinterName(printerIndex)) == 0;
	}

	public void shutDown()
	{
	}
	
	class TestPage implements Printable
	{
		public int print(Graphics g, PageFormat format, int index) throws PrinterException
		{
			g.setFont(new Font("Arial", Font.PLAIN, 12));
			g.drawString("Hello, World!", (int)(format.getImageableX() + format.getImageableWidth() * 0.1), (int)(format.getImageableY() + format.getImageableHeight() * 0.1));
			return PAGE_EXISTS;
		}
	}
}
