package uremoteagents;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.util.*;
import java.net.InetAddress;
import uremote.*;

/**
 * @author Greg Hackmann
 */
public class ScreenshotAgent implements DeviceDefinition
{
	private String ip;
	private Robot bot;
	private GenericDeviceAgent agent;
	private Dimension screenSize;
	private Rectangle zoomRect;
	private Hashtable adTable, helpTable;
	
	private String [] functions = { "Up", "Down", "Left", "Right", "ZoomIn", "ZoomOut", "Snap", "Screenshot" };
	
	public ScreenshotAgent()
	{
		try
		{
			bot = new Robot();
		}
		catch(AWTException e)
		{
			e.printStackTrace();
		}

		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		screenSize  = Toolkit.getDefaultToolkit().getScreenSize();
		zoomRect = new Rectangle(0, 0, screenSize.width, screenSize.height);
		
		adTable = new Hashtable();
		adTable.put("Up", new ButtonAdvertisement("Up", false, "ARROWUP", 2, 0));
		adTable.put("Down", new ButtonAdvertisement("Down", false, "ARROWDOWN", 2, 3));
		adTable.put("Left", new ButtonAdvertisement("Left", false, "ARROWLEFT", 0, 1, 1, 2));
		adTable.put("Right", new ButtonAdvertisement("Right", false, "ARROWRIGHT", 4, 1, 1, 2));
		adTable.put("ZoomIn", new ButtonAdvertisement("ZoomOut", false, "REWIND", 0, 3));
		adTable.put("ZoomOut", new ButtonAdvertisement("ZoomIn", false, "FASTFORWARD", 4, 3));
		adTable.put("Snap", new ButtonAdvertisement("Snap", false, "RECORD", 4, 0));
		adTable.put("Screenshot", new ImageAdvertisement("Screenshot", 1, 1, 3, 2));

		helpTable = new Hashtable();
		helpTable.put("Up", "Scroll up");
		helpTable.put("Down", "Scroll down");
		helpTable.put("Left", "Scroll left");
		helpTable.put("Right", "Scroll right");
		helpTable.put("ZoomIn", "Zoom in");
		helpTable.put("ZoomOut", "Zoom out");
		helpTable.put("Snap", "Update the screenshot");
		helpTable.put("Screenshot", "");
	}
	
	public String getName()
	{
		return "Screenshot";
	}

	public String getID()
	{
		return getName() + ip;
	}

	public String getIcon()
	{
		return "TV";
	}
	
	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("ZoomIn"))
		{
			double w = zoomRect.getWidth(), h = zoomRect.getHeight();
			zoomRect = new Rectangle((int)(zoomRect.x + w / 4), (int)(zoomRect.y + h / 4), (int)(w / 2), (int)(h / 2));
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("ZoomOut"))
		{
			double w = zoomRect.getWidth() * 2, h = zoomRect.getHeight() * 2;
			if(w > screenSize.width) w = screenSize.width;
			if(h > screenSize.height) h = screenSize.height;
			
			double x = zoomRect.x - w / 4, y = zoomRect.y - h / 4;
			if(x < 0) x = 0;
			if(y < 0) y = 0;
			if(x + w > screenSize.width) x = screenSize.width - w;
			if(y + h > screenSize.height) y = screenSize.height - h;
			
			zoomRect = new Rectangle((int)x, (int)y, (int)w, (int)h);
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("Left"))
		{
			double x = zoomRect.x - zoomRect.width;
			if(x < 0) x = 0;
			zoomRect.x = (int)x;
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("Right"))
		{
			double x = zoomRect.x + zoomRect.width;
			if(x + zoomRect.width > screenSize.width) x = screenSize.width - zoomRect.width;
			zoomRect.x = (int)x;
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("Up"))
		{
			double y = zoomRect.y - zoomRect.height;
			if(y < 0) y = 0;
			zoomRect.y = (int)y;
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("Down"))
		{
			double y = zoomRect.x + zoomRect.height;
			if(y + zoomRect.height > screenSize.height) y = screenSize.height - zoomRect.height;
			zoomRect.y = (int)y;
			agent.changeState("Screenshot", getState("Screenshot"));
		}
		else if(functionName.equals("Snap"))
		{
			agent.changeState("Screenshot", getState("Screenshot"));
		}
	}

	public void reactToSlider(String dummy, int dummy2)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}
	
	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}
	
	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(!functionName.equals("Screenshot")) return new ButtonState(true, false);
		
		BufferedImage image = bot.createScreenCapture(zoomRect);
		
		double xScale = 144.0 / image.getWidth(), yScale = 96.0 / image.getHeight();
		AffineTransform scaleOp = AffineTransform.getScaleInstance(xScale, yScale);
		AffineTransformOp scaler = new AffineTransformOp(scaleOp, AffineTransformOp.TYPE_BILINEAR);
		BufferedImage scaled = scaler.filter(image, null);
		
		return new ImageState(scaled, "Screenshot");
	}

	public void shutDown()
	{
	}
}
