package uremoteagents;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Hashtable;

import javax.swing.*;
import uremote.*;

public class WeatherAgent implements DeviceDefinition
{
	private GenericDeviceAgent agent;
	private String ip;
	private JTextField zipCode;
	
	private static final String header1 = 
		"POST /usweather.asmx HTTP/1.1\n" +
		"Host: www.webservicex.net\n" +
		"Content-Type: text/xml; charset=utf-8\n" +
		"Content-Length: ";
	
	private static final String header2 = "\nSOAPAction: \"http://www.webserviceX.NET/GetWeatherReport\"\n\n";
		
	private static final String content1 =
		"<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
		"<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
		"<soap:Body>" +
		"<GetWeatherReport xmlns=\"http://www.webserviceX.NET\">" +
		"<ZipCode>";
	
	private static final String content2 = 	
		"</ZipCode></GetWeatherReport></soap:Body></soap:Envelope>\n\n";
	
	private String fahrenheit = "???", celsius = "???", humidity = "???",
	                condition = "???", wind = "???";
	
	private Hashtable adTable, helpTable;
	private String functions [] = { "Temperature", "Humidity", "Condition", "Wind", "Update" };
		
	public WeatherAgent()
	{
		adTable = new Hashtable();
		
		adTable.put("Update", new ButtonAdvertisement("Update", false, "RECORD", 4, 0, 1, 4));
		adTable.put("Temperature", new LabelAdvertisement("Temperature", 0, 0, 4, 1));
		adTable.put("Humidity", new LabelAdvertisement("Humidity", 0, 1, 4, 1));
		adTable.put("Condition", new LabelAdvertisement("Condition", 0, 2, 4, 1));
		adTable.put("Wind", new LabelAdvertisement("Wind", 0, 3, 4, 1));
		
		helpTable = new Hashtable();
		
		helpTable.put("Update", "Update the weather readout");
		helpTable.put("Temperature", "");
		helpTable.put("Humidity", "");
		helpTable.put("Condition", "");
		helpTable.put("Wind", "");
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("WeatherAgent");
		JPanel contents = new JPanel(new FlowLayout());
		contents.add(new JLabel("ZIP Code:"));
		
		zipCode = new JTextField("63105", 5);
		contents.add(zipCode);
		frame.setContentPane(contents);
		
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
		
		new Thread()
		{
			public void run()
			{
				updateWeather();
			}
		}.start();
	}

	private void updateWeather()
	{
		try
		{
			Socket http = new Socket("www.webserviceX.net", 80);
				
			String content = content1 + zipCode.getText() + content2;
			String header = header1 + content.length() + header2;
			DataOutputStream output = new DataOutputStream(http.getOutputStream());
			BufferedReader input = new BufferedReader(new InputStreamReader(http.getInputStream()));	
	
			output.writeBytes(header);
	
			while(!input.readLine().equals("")) { }
	
			output.writeBytes(content);
			
			String line;
			int contentLength = -1;
			while(!(line = input.readLine()).equals(""))
			{
				if(line.startsWith("Content-Length: "))
					contentLength = Integer.parseInt(line.substring(new String("Content-Length: ").length()));
			}
	
			if(contentLength == -1) return;
			
			char [] buffer = new char[contentLength];
			int bytesLeft = contentLength;
	
			while(bytesLeft > 0)
				bytesLeft -= input.read(buffer, contentLength - bytesLeft, bytesLeft);
	
			http.close();
	
			String result = new String(buffer);
			result = result.replaceAll("&gt;", ">");
			result = result.replaceAll("&lt;", "<");
			
			fahrenheit = result.split("</?Fahrenheit>")[1];
			celsius = result.split("</?Celsius>")[1];
			humidity = result.split("</?Humidity>")[1];
			condition = result.split("</?Condition>")[1];
			wind = result.split("</?Wind>")[1];
			
			agent.changeState(new String [] { "Temperature", "Humidity", "Condition", "Wind" },
				              new ComponentState [] { getState("Temperature"),
				              					      getState("Humidity"),
				              					      getState("Condition"),
				              					      getState("Wind") });

		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public String getName()
	{
		return "Weather";
	}

	public String getID()
	{
		return getName() + ip;
	}

	public String getIcon()
	{
		return "TV";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		new Thread()
		{
			public void run()
			{
				updateWeather();
			}
		}.start();
	}

	public void reactToSlider(String functionName, int value)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Temperature"))
		{
			return new LabelState(true, "Temperature: " + fahrenheit + "/" + celsius);
		}
		else if(functionName.equals("Humidity"))
		{
			return new LabelState(true, "Humidity: " + humidity);
		}
		else if(functionName.equals("Condition"))
		{
			return new LabelState(true, "Condition: " + condition);
		}
		else if(functionName.equals("Wind"))
		{
			return new LabelState(true, "Wind: " + wind);
		}
		else return new ButtonState(true, false);
	}

	public void shutDown()
	{
	}	
}