package uremoteagents;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.InetAddress;
import java.util.Hashtable;
import jhwnd.JHWnd;
import uremote.*;
import java.util.*;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

public class WinampAgent implements DeviceDefinition
{
	private static final int PREVIOUS_TRACK = 40044;
	private static final int NEXT_TRACK = 40048;
	private static final int PLAY = 40045;
	private static final int PAUSE = 40046;
	private static final int STOP = 40047;
	private static final int FAST_FORWARD = 40148;
	private static final int REWIND = 40144;
	private static final int VOLUME_UP = 40058;
	private static final int VOLUME_DOWN = 40059;
	
	private static final int STATUS = 104;
	private static final int TRACK_STATUS = 105;
	private static final int SEEK_TO = 106;
	private static final int SET_VOLUME = 122;
	
	private Hashtable adTable, helpTable;
	private String functions [] = { "Song Title", "Play", "Stop", "Previous",
									 "Next", "FastForward", "Rewind", "Pause", "Volume" };
                                     
    private GenericDeviceAgent agent;
    private RefreshTask task;
    private String ip;
	
	static
	{
		System.loadLibrary("JHWnd");
	}
	
	private int hWnd = 0;
	private int x = 0, y = 1;
	
	public WinampAgent()
	{
		hWnd = JHWnd.findWindow("Winamp v1.x");
		if(hWnd == 0)
		{
			JOptionPane.showMessageDialog(null, "Winamp is not running.", "Could not load agent", JOptionPane.ERROR_MESSAGE);
			return;
		}
		adTable = new Hashtable();
		
		adTable.put("Song Title", new SliderAdvertisement("Song Title", 0, 100, 0, 0, 5, 1));
		adTable.put("Volume", new SliderAdvertisement("Volume", 0, 255, 0, 1, 5, 1));
		adTable.put("Previous", new ButtonAdvertisement("Previous", false, "PREVIOUS", 0, 3));
		adTable.put("Rewind", new ButtonAdvertisement("Rewind", false, "REWIND", 1, 3));
		adTable.put("FastForward", new ButtonAdvertisement("FastForward", false, "FASTFORWARD", 3, 3));
		adTable.put("Next", new ButtonAdvertisement("Next", false, "NEXT", 4, 3));
		adTable.put("Play", new ButtonAdvertisement("Play", false, "PLAY", 0, 2, 2, 1));
		adTable.put("Pause", new ButtonAdvertisement("Pause", false, "PAUSE", 2, 2));
		adTable.put("Stop", new ButtonAdvertisement("Stop", false, "STOP", 3, 2, 2, 1));
		
		helpTable = new Hashtable();
		
		helpTable.put("Song Title", "");
		helpTable.put("Volume", "");
		helpTable.put("Previous", "Go to the previous track");
		helpTable.put("Rewind", "Go back 5 seconds");
		helpTable.put("FastForward", "Go ahead 5 seconds");
		helpTable.put("Next", "Go to the next track");
		helpTable.put("Play", "Play the current track");
		helpTable.put("Pause", "Pause or unpause the current track");
		helpTable.put("Stop", "Stop the current track");
		
		final Timer refresh = new Timer(false);
		task = new RefreshTask();
		refresh.scheduleAtFixedRate(task, 0, 5000);
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("WinampAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				refresh.cancel();
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	class RefreshTask extends TimerTask
	{
		public void run()
		{
			if(agent == null) return;
			
			String functions [] = new String []
			{
				"Pause",
				"Stop",
				"Song Title",
				"Volume"
			};
			ComponentState states [] = new ComponentState[functions.length];
			
			for(int i = 0; i < functions.length; i++) states[i] = getState(functions[i]);
			agent.changeState(functions, states);
		}
	}
	
	private long sendCommand(int cmd)
	{
		return JHWnd.sendMessage(hWnd, JHWnd.WM_COMMAND, cmd, 0);
	}
	
	private long sendUser(int cmd)
	{
		return sendUser(0, cmd);
	}
	
	private long sendUser(int data, int cmd)
	{
		return JHWnd.sendMessage(hWnd, JHWnd.WM_USER, data, cmd);
	}
	
	private long status() { return sendUser(STATUS); }
	
	public boolean isPlaying() { return status() == 1; }
	public boolean isPaused() { return status() == 3; }
	public boolean isStopped() { return !isPlaying() && !isPaused(); }
	
	public void previousTrack() { sendCommand(PREVIOUS_TRACK); }
	public void nextTrack() { sendCommand(NEXT_TRACK); }
	
	public void play() { sendCommand(PLAY); }
	public void pause() { sendCommand(PAUSE); }
	public void stop() { sendCommand(STOP); }
	
	public void fastForward() { sendCommand(FAST_FORWARD); }
	public void rewind() { sendCommand(REWIND); }
	
	public void volumeUp() { sendCommand(VOLUME_UP); }
	public void volumeDown() { sendCommand(VOLUME_DOWN); }

	public long getTrackPosition() { return sendUser(0, TRACK_STATUS); }
	public void setTrackPosition(int ms) { sendUser(ms, SEEK_TO); }
	public long getTrackLength() { return sendUser(1, TRACK_STATUS); }
	
	/* Getting the volume is done by *setting* the volume to -666 and seeing
	   what it returns.  No, this is not documented in the Winamp SDK. */
	public long getVolume() { return sendUser(-666, SET_VOLUME); }
	public void setVolume(int volume) { sendUser(volume, SET_VOLUME); }

	public String getTrackTitle()
	{
		String windowTitle = JHWnd.getWindowText(hWnd);
		int indexOfWinamp = windowTitle.lastIndexOf(" - Winamp");
		if(indexOfWinamp == -1) return "";
		
		return windowTitle.substring(0, indexOfWinamp);
	}

	public String getName()
	{
		return "Winamp";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "CD";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Play")) play();
		else if(functionName.equals("Stop")) stop();
		else if(functionName.equals("Pause")) pause();
		else if(functionName.equals("Previous")) previousTrack();
		else if(functionName.equals("Next")) nextTrack();
		else if(functionName.equals("Rewind")) rewind();
		else if(functionName.equals("FastForward")) fastForward();
		
		task.run();
	}

	public void reactToSlider(String functionName, int value)
	{
		if(functionName.equals("Volume"))setVolume(value);
		else if(functionName.equals("Song Title")) setTrackPosition((int)(getTrackLength() * value * 10));
		
		task.run();
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Volume"))
		{
			return new SliderState(true, "Volume", (int)getVolume());
		}
		else if(functionName.equals("Song Title"))
		{
			return new SliderState(!isStopped(), getTrackTitle(), (int)(getTrackPosition() / getTrackLength() / 10));
		}
		else if(functionName.equals("Pause"))
		{
			return new ButtonState(!isStopped(), isPaused());
		}
		else if(functionName.equals("Stop"))
		{
			return new ButtonState(!isStopped(), false);
		}
		return new ButtonState(true, false);
	}

	public void shutDown()
	{
	}
}
