package uremoteagents;

import javax.swing.*;
import javax.comm.*;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;
import java.util.*;
import com.jpeterson.x10.*;
import com.jpeterson.x10.event.*;
import com.jpeterson.x10.module.*;
import uremote.*;

public class X10Agent implements DeviceDefinition, FunctionListener
{
	private String ip;
	private String [] functions = new String [] { "Brighter", "Dimmer", "Power" };
	private Hashtable helpTable, adTable;
	private GenericDeviceAgent agent;
	
	private static final boolean fake = false;
	private JFrame frame;
	private JComboBox houseCombo, unitCombo;
	private int brightness = 100;
	private boolean lit = false;
	private static CM11A x10 = null;
	private static int agentCount = 0;
	private static Object agentCountLock = new Object();
	
	public X10Agent()
	{
		adTable = new Hashtable();
		adTable.put("Brighter", new ButtonAdvertisement("Brighter", false, "ARROWUP", 3, 1));
		adTable.put("Dimmer", new ButtonAdvertisement("Dimmer", false, "ARROWDOWN", 3, 2));
		adTable.put("Power", new ButtonAdvertisement("Power", false, "POWER", 1, 1, 2, 2));
		
		helpTable = new Hashtable();
		helpTable.put("Brighter", "Brighten the light");
		helpTable.put("Dimmer", "Dim the light");
		helpTable.put("Power", "Turn the light on or off");
		
		if(fake)
		{
			frame = new JFrame("The world's least energy-efficient light");
			frame.getContentPane().setBackground(Color.white);
			frame.show();
			frame.setExtendedState(Frame.MAXIMIZED_BOTH);
			frame.addWindowListener(new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					agent.closeAgent();
				}
			});
		}
		else
		{
			synchronized(agentCountLock)
			{
				agentCount++;
			}

			frame = new JFrame("X10 Agent");
			
			if(x10 == null)
			{
				final JDialog dialog = new JDialog(frame, "Select X10 port", true);
				JPanel panel = new JPanel(new BorderLayout(10, 10));
				
				JPanel labelPanel = new JPanel(new BorderLayout());
				labelPanel.add(new JLabel("Please select the serial port that the X10"), BorderLayout.NORTH);
				labelPanel.add(new JLabel("interface is attached to."), BorderLayout.SOUTH);
				panel.add(labelPanel, BorderLayout.NORTH);
				
				JPanel x10Panel = new JPanel();
				
				Enumeration enum = CommPortIdentifier.getPortIdentifiers();
				Vector enumVec = new Vector();
				while(enum.hasMoreElements())
				{
					CommPortIdentifier current = (CommPortIdentifier)enum.nextElement();
					if(current.getPortType() == CommPortIdentifier.PORT_SERIAL) enumVec.add(current);
				}
		
				String [] ports = new String[enumVec.size()];
				for(int i = 0; i < enumVec.size(); i++)
					ports[i] = ((CommPortIdentifier)enumVec.get(i)).getName();
							
				JComboBox portCombo = new JComboBox(ports);
				x10Panel.add(portCombo);
				
				panel.add(x10Panel, BorderLayout.CENTER);
				
				JButton button = new JButton("OK");
				button.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						dialog.hide();
					}
				});
				button.setDefaultCapable(true);
				panel.add(button, BorderLayout.SOUTH);
				dialog.getRootPane().setDefaultButton(button);
				
				dialog.setContentPane(panel);
				dialog.setResizable(false);
				dialog.pack();
				dialog.show();
				
				String portName = (String)portCombo.getSelectedItem();
				x10 = new CM11A();
				x10.setPortName(portName);
				x10.addFunctionListener(this);
	
				try
				{
					x10.allocate();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			
			JPanel panel = new JPanel(new BorderLayout(10, 10));
			
			JPanel labelPanel = new JPanel(new BorderLayout());
			labelPanel.add(new JLabel("Please select the device and house code of the"), BorderLayout.NORTH);
			labelPanel.add(new JLabel("X10 device to be controlled."), BorderLayout.SOUTH);
			panel.add(labelPanel, BorderLayout. NORTH);
			
			JPanel x10Panel = new JPanel();
			
			String [] houseCodes = new String['P' - 'A' + 1];
			for(char c = 'A'; c <= 'P'; c++) houseCodes[c - 'A'] = "" + c;
	
			houseCombo = new JComboBox(houseCodes);
			x10Panel.add(houseCombo);

			String [] unitCodes = new String[16];
			for(int i = 0; i < 16; i++) unitCodes[i] = "" + (i + 1);
	
			unitCombo = new JComboBox(unitCodes);
			x10Panel.add(unitCombo);
			panel.add(x10Panel, BorderLayout.SOUTH);
			
			frame.setContentPane(panel);
			frame.pack();
			frame.setResizable(false);
			frame.addWindowListener(new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					agent.closeAgent();
				}
			});
			frame.show();
		}
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private char getHouseCode()
	{
		return ((String)houseCombo.getSelectedItem()).charAt(0);
	}
	
	private int getUnitCode()
	{
		return Integer.parseInt(((String)unitCombo.getSelectedItem()));
	}
	
	private void sendX10(X10Event event)
	{
		X10Event [] events = new X10Event[2];
		Transmitter transmitter = (Transmitter)x10;
		
		try
		{
			transmitter.transmit(new AddressEvent(this, getHouseCode(), getUnitCode()));
			transmitter.transmit(event);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	private void updateColor()
	{
		int scaled;
		
		if(lit) scaled = (int)((255.0 / 100.0) * brightness);
		else scaled = 0;
		
		frame.getContentPane().setBackground(new Color(scaled, scaled, scaled));
	}
	
	private void brighten()
	{
		on();
		
		if(fake)
		{
			brightness += 5;
			if(brightness > 100) brightness = 100;
			
			updateColor();
		}
		else
		{
			sendX10(new BrightEvent(this, getHouseCode(), 1, 22));
		}
	}

	private void dim()
	{
		on();
		
		if(fake)
		{
			brightness -= 5;
			if(brightness < 0) brightness = 0;
			
			updateColor();
		}
		else
		{
			sendX10(new DimEvent(this, getHouseCode(), 1, 22));
		}
	}
	
	private void on()
	{
		if(fake)
		{
			lit = true;
			updateColor();
		}
		else
		{
			sendX10(new OnEvent(this, getHouseCode()));
		}
	}
	
	private void off()
	{
		if(fake)
		{
			lit = false;
			updateColor();
		}
		else
		{
			sendX10(new OffEvent(this, getHouseCode()));
		}
	}

	public String getName()
	{
		return "X10";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "X10";
	}
	
	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}
	
	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Brighter")) brighten();
		else if(functionName.equals("Dimmer")) dim();
		else if(functionName.equals("Power"))
		{
			if(lit) off();
			else on();
		} 
	}
	
	public void reactToSlider(String functionName, int value)
	{
	}
	
	public String[] getFunctions()
	{
		return functions;
	}
	
	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}
	
	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Power"))
			return new ButtonState(true, lit);
		else return new ButtonState(true, false);
	}

	public void shutDown()
	{
		if(!fake && x10 != null)
		{
			int currentCount;

			synchronized(agentCountLock)
			{
				currentCount = --agentCount;
			}

			if(currentCount == 0)
			{
				try
				{
					x10.deallocate();
					x10 = null;
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	public void functionAllLightsOff(FunctionEvent e)
	{
	}

	public void functionAllLightsOn(FunctionEvent e)
	{
	}

	public void functionAllUnitsOff(FunctionEvent e)
	{
	}

	public void functionBright(FunctionEvent e)
	{
	}

	public void functionDim(FunctionEvent e)
	{
	}

	public void functionHailAcknowledge(FunctionEvent e)
	{
	}

	public void functionHailRequest(FunctionEvent e)
	{
	}

	public void functionOff(FunctionEvent e)
	{
		lit = false;
		agent.changeState("Power", getState("Power"));
	}

	public void functionOn(FunctionEvent e)
	{
		lit = true;
		agent.changeState("Power", getState("Power"));
	}

	public void functionPresetDim1(FunctionEvent e)
	{
	}

	public void functionPresetDim2(FunctionEvent e)
	{
	}

	public void functionStatusOff(FunctionEvent e)
	{
	}

	public void functionStatusOn(FunctionEvent e)
	{
	}

	public void functionStatusRequest(FunctionEvent e)
	{
	}
}
