package uremoteagents;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import com.apple.cocoa.foundation.*;
import uremote.*;
import limone.util.*;
import javax.swing.*;

public class iPhotoAgent implements DeviceDefinition
{
	private String ip;
	private GenericDeviceAgent agent;
	private Rectangle zoomRect;
	private Hashtable adTable, helpTable;
	private int current = 0;
	private Toolkit kit;
	
	private String [] functions = { "Previous", "Next", "Photo" };
	
	public static void main(String [] args)
	{
		Launcher.main(new String [] { "-newServer", "-mPort", "2223", "-sPort", "2222" } );
		Launcher.main(new String [] { "-load", "uremote.GenericDeviceAgent", "-name", "uremoteagents.iPhotoAgent", "-noExit", "-mPort", "2223", "-sPort", "2222" } );
	}
	
	public iPhotoAgent()
	{
		kit = Toolkit.getDefaultToolkit();
		com.apple.cocoa.application.NSApplication.sharedApplication();
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		adTable = new Hashtable();
		adTable.put("Previous", new ButtonAdvertisement("Previous", false, "REWIND", 0, 3));
		adTable.put("Next", new ButtonAdvertisement("Next", false, "FASTFORWARD", 4, 3));
//		adTable.put("Photo", new ButtonAdvertisement("Photo", false, "POWER", 1, 1, 3, 2));
		adTable.put("Photo", new ImageAdvertisement("Photo", 1, 0, 3, 4));

		helpTable = new Hashtable();
		helpTable.put("Previous", "View the previous photo");
		helpTable.put("Next", "View the next photo");
		helpTable.put("Photo", "");
		
		JFrame frame = new JFrame("iPhotoAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	private NSAppleEventDescriptor script(String text)
	{
		text = "tell application \"iPhoto\"\n" + text + "\nend tell";
		return new NSAppleScript(text).execute(new NSMutableDictionary());
	}
	
	private void nextPhoto()
	{
		if(current >= numPhotos() - 1) current = 0;
		else current++;
	}
	
	private void previousPhoto()
	{
		if(current == 0) current = numPhotos() - 1;
		else current--;
	}
	
	private int numPhotos()
	{
		return script("get count of photos").int32Value();
	}
	
	
	public String getName()
	{
		return "iPhoto";
	}

	public String getID()
	{
		return getName() + ip;
	}

	public String getIcon()
	{
		return "TV";
	}
	
	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}

	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Next")) nextPhoto();
		else if(functionName.equals("Previous")) previousPhoto();
		
		agent.changeState("Photo", getState("Photo"));
	}

	public void reactToSlider(String dummy, int dummy2)
	{
	}

	public String[] getFunctions()
	{
		return functions;
	}
	
	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}
	
	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(!functionName.equals("Photo")) return new ButtonState(true, false);
		String filename = script("get image path of photo " + (current + 1)).stringValue();
		
		Image image = kit.getImage(filename);
		MediaTracker mediaTracker = new MediaTracker(new Container());
    	mediaTracker.addImage(image, 0);
    	try { mediaTracker.waitForID(0); } catch(Exception e) {  e.printStackTrace(); }
		int width = image.getWidth(null), height = image.getHeight(null);
		
		BufferedImage buffered = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		buffered.createGraphics().drawImage(image, 0, 0, null);
		
		double xScale = 144.0 / width, yScale = 192.0 / height;
		AffineTransform scaleOp = AffineTransform.getScaleInstance(xScale, yScale);
		AffineTransformOp scaler = new AffineTransformOp(scaleOp, AffineTransformOp.TYPE_BILINEAR);
		BufferedImage scaled = scaler.filter(buffered, null);
		
		return new ImageState(scaled, "Photo");
	}

	public void shutDown()
	{
	}
}
