package uremoteagents;

import java.util.*;
import java.net.InetAddress;
import javax.swing.JLabel;
import javax.swing.JFrame;

import java.awt.event.*;
import com.apple.cocoa.foundation.*;
import uremote.*;

public class iTunesAgent implements DeviceDefinition
{
	private String functions [] = { "Song Title", "Play", "Previous", "Mute",
									 "Next", "FastForward", "Rewind", "Volume" };
	private Hashtable adTable, helpTable;
	private RefreshTask task;
	private String ip;
	private GenericDeviceAgent agent;
	
	public iTunesAgent()
	{
		com.apple.cocoa.application.NSApplication.sharedApplication();
		
		adTable = new Hashtable();
		
		adTable.put("Song Title", new SliderAdvertisement("Song Title", 0, 100, 0, 0, 5, 1));
		adTable.put("Volume", new SliderAdvertisement("Volume", 0, 100, 0, 1, 4, 1));
		adTable.put("Mute", new ButtonAdvertisement("Mute", false, "MUTE", 4, 1));
		adTable.put("Previous", new ButtonAdvertisement("Previous", false, "PREVIOUS", 0, 3));
		adTable.put("Rewind", new ButtonAdvertisement("Rewind", false, "REWIND", 1, 3));
		adTable.put("FastForward", new ButtonAdvertisement("FastForward", false, "FASTFORWARD", 3, 3));
		adTable.put("Next", new ButtonAdvertisement("Next", false, "NEXT", 4, 3));
		adTable.put("Play", new ButtonAdvertisement("Play", false, "PLAY", 2, 3));
		
		helpTable = new Hashtable();
		
		helpTable.put("Song Title", "");
		helpTable.put("Volume", "");
		helpTable.put("Mute", "Mute or unmute the sound");
		helpTable.put("Previous", "Go to the previous track");
		helpTable.put("Rewind", "Go back 5 seconds");
		helpTable.put("FastForward", "Go ahead 5 seconds");
		helpTable.put("Next", "Go to the next track");
		helpTable.put("Play", "Play or pause the current track");
		
		final Timer refresh = new Timer(false);
		task = new RefreshTask();
		refresh.scheduleAtFixedRate(task, 0, 5000);
		
		try
		{
			ip = InetAddress.getLocalHost().getHostAddress();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		JFrame frame = new JFrame("iTunesAgent");
		frame.getContentPane().add(new JLabel("Close this window to close the agent"));
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				refresh.cancel();
				agent.closeAgent();
			}
		});
		frame.pack();
		frame.show();
	}
	
	class RefreshTask extends TimerTask
	{
		public void run()
		{
			if(agent == null) return;
			
			String functions [] = new String []
			{
				"Play",
				"Mute",
				"Song Title",
				"Volume"
			};
			ComponentState states [] = new ComponentState[functions.length];
			
			for(int i = 0; i < functions.length; i++) states[i] = getState(functions[i]);
			agent.changeState(functions, states);
		}
	}

	private NSAppleEventDescriptor script(String text)
	{
		text = "tell application \"iTunes\"\n" + text + "\nend tell";
		return new NSAppleScript(text).execute(new NSMutableDictionary());
	}
	
	public void play()
	{
		script("if player state is stopped then\n" +
				   "play\n" +
		       "else\n" +
				   "pause\n" +
			   "end if");
	}
	
	public void mute()
	{
		script("set mute to not mute");
	}
	
	public void previousTrack()
	{
		script("set current track to previous track");
	}

	public void nextTrack()
	{
		script("set current track to next track");
	}
	
	public void rewind()
	{
		script("set player position to player position - 5");
	}
	
	public void fastForward()
	{
		script("set player position to player position + 5");
	}
	
	public void setVolume(int volume)
	{
		script("set sound volume to " + volume);
	}
	
	public int getVolume()
	{
		return script("get sound volume").int32Value();
	}
	
	public boolean isPlaying()
	{
		return script("return player state is playing").booleanValue();
	}
	
	public boolean isMuted()
	{
		return script("get mute").booleanValue();
	}
	
	public String getTrackTitle()
	{
		String title = script("get name of current track").stringValue();
		return (title == null ? "" : title);
	}
	
	public int getTrackPosition()
	{
		return script("get player position").int32Value();
	}
	
	public void setTrackPosition(int value)
	{
		script("set player position to " + value);
	}
	
	public int getTrackLength()
	{
		return script("get finish of current track").int32Value();
	}
	
	public String getName()
	{
		return "iTunes";
	}
	
	public String getID()
	{
		return getName() + ip;
	}
	
	public String getIcon()
	{
		return "CD";
	}

	public void setAgent(GenericDeviceAgent agent)
	{
		this.agent = agent;
	}
	
	public void reactToButton(String functionName, boolean toggle)
	{
		if(functionName.equals("Play")) play();
		else if(functionName.equals("Mute")) mute();
		else if(functionName.equals("Previous")) previousTrack();
		else if(functionName.equals("Next")) nextTrack();
		else if(functionName.equals("Rewind")) rewind();
		else if(functionName.equals("FastForward")) fastForward();
		
		task.run();
	}

	public void reactToSlider(String functionName, int value)
	{
		if(functionName.equals("Volume")) setVolume(value);
		else if(functionName.equals("Song Title")) setTrackPosition((int)(getTrackLength() / 100.0 * value));
		
		task.run();
	}

	public String[] getFunctions()
	{
		return functions;
	}

	public String getHelpText(String functionName)
	{
		return (String)helpTable.get(functionName);
	}

	public ComponentAdvertisement getAdvertisement(String functionName)
	{
		return (ComponentAdvertisement)adTable.get(functionName);
	}

	public ComponentState getState(String functionName)
	{
		if(functionName.equals("Volume"))
		{
			return new SliderState(true, "Volume", getVolume());
		}
		else if(functionName.equals("Song Title"))
		{
			return new SliderState(true, getTrackTitle(), (int)(100.0 * getTrackPosition() / getTrackLength()));
		}
		else if(functionName.equals("Mute"))
		{
			return new ButtonState(true, isMuted());
		}
		else if(functionName.equals("Play"))
		{
			return new ButtonState(true, isPlaying());
		}
		return new ButtonState(true, false);
	}

	public void shutDown()
	{
	}
}
