#include "JWIA.h"
#include <jawt.h>
#include <jawt_md.h>
#include <windows.h>
#include <wia.h>
#include <wiavideo.h>
#include <sti.h>
#include <oleauto.h>
#include <initguid.h>

DEFINE_GUID(IID_IWiaDevMgr, 0x5EB2502A, 0x8CF1, 0x11D1, 0xBF, 0x92, 0x00, 0x60, 0x08, 0x1E, 0xD8, 0x11);
DEFINE_GUID(CLSID_WiaDevMgr, 0xA1F4E726, 0x8CF1, 0x11D1, 0xBF, 0x92, 0x00, 0x60, 0x08, 0x1E, 0xD8, 0x11);
DEFINE_GUID(IID_IWiaVideo, 0xD52920AA, 0xDB88, 0x41F0, 0x94, 0x6C, 0xE0, 0x0D, 0xC0, 0xA1, 0x9C, 0xFA);
DEFINE_GUID(CLSID_WiaVideo, 0x3908C3CD, 0x4478, 0x4536, 0xAF, 0x2F, 0x10, 0xC2, 0x5D, 0x4E, 0xF8, 0x9A);
DEFINE_GUID(IID_IWiaPropertyStorage, 0x98B5E8A0, 0x29CC, 0x491A, 0xAA, 0xC0, 0xE6, 0xDB, 0x4F, 0xDC, 0xCE, 0xB6);
/* Normally you get these CLSIDs by linking with a .LIB file.  We're using gcc, so we don't
   have that option. */

const int PRSPEC_PROPID	= 1;

JNIEXPORT jint JNICALL Java_jwia_JWIA_openManager(JNIEnv * env, jclass j_class)
{
	IWiaDevMgr * manager = NULL;

	CoInitializeEx(NULL, COINIT_MULTITHREADED);
	/* Initialize COM */
	
	CoCreateInstance(&CLSID_WiaDevMgr, NULL, CLSCTX_LOCAL_SERVER, &IID_IWiaDevMgr, (void * *)&manager);
	/* Get an instance of the WIA device manager */
	
	return (int)manager;
}

JNIEXPORT void JNICALL Java_jwia_JWIA_closeManager(JNIEnv * env, jclass j_class, jint manager_int)
{
	IWiaDevMgr * manager = (IWiaDevMgr *)manager_int;
	manager->lpVtbl->Release(manager);
}

JNIEXPORT jint JNICALL Java_jwia_JWIA_openCamera(JNIEnv * env, jclass j_class, jint manager_int)
{
	IWiaDevMgr * manager = (IWiaDevMgr *)manager_int;
	IEnumWIA_DEV_INFO * devices = NULL;
	BSTR id = NULL;
	
	if(!SUCCEEDED(manager->lpVtbl->EnumDeviceInfo(manager, WIA_DEVINFO_ENUM_LOCAL, &devices))) return 0;
	devices->lpVtbl->Reset(devices);
	/* Get an enumeration of all the WIA devices on the system */
	
	IWiaPropertyStorage * properties = NULL;
	ULONG num;
	
	/* Keep going until we either give up or find one */
	while(1)
	{
		devices->lpVtbl->Next(devices, 1, &properties, &num);
		/* Get the next device */
		if(num == 0)
		{
			devices->lpVtbl->Release(devices);
			return 0;
		}
		/* If it failed, give up */

		PROPSPEC specs[2];
		PROPVARIANT variants[2];

		memset(variants, 0, 2 * sizeof(PROPVARIANT));
		specs[0].ulKind = PRSPEC_PROPID;
		specs[0].propid = WIA_DIP_DEV_TYPE;
		/* Property 0 = device type */
		specs[1].ulKind = PRSPEC_PROPID;
		specs[1].propid = WIA_DIP_DEV_ID;
        /* Property 1 = device ID */
		
		properties->lpVtbl->ReadMultiple(properties, 2, specs, variants);
		/* Read in the properties */

		if(GET_STIDEVICE_TYPE(variants[0].lVal) == StiDeviceTypeStreamingVideo)
		{
			id = variants[1].bstrVal;
			goto found;
		}
		/* If we've got a streaming video device, then remember the ID and leave
		   the loop */

		properties->lpVtbl->Release(properties);
		/* Free the memory and go on to the next device */
	}
	
found:
	devices->lpVtbl->Release(devices);
	properties->lpVtbl->Release(properties);
	/* We don't need these anymore */

	IWiaItem * item = NULL;
	manager->lpVtbl->CreateDevice(manager, id, &item);
	/* Now we need to get a device from the ID so we can set its image directory */
	
	if(!SUCCEEDED(properties->lpVtbl->QueryInterface(properties, &IID_IWiaPropertyStorage, (void * *)&properties))) return 0;
	/* Get the device's properties */
	
	PROPSPEC spec;
	PROPVARIANT variant;

	memset(&variant, 0, sizeof(PROPVARIANT));
	spec.ulKind = PRSPEC_PROPID;
	spec.propid = WIA_DPV_IMAGES_DIRECTORY;
	/* We want to look up the default image directory */
	
	properties->lpVtbl->ReadMultiple(properties, 1, &spec, &variant);
	properties->lpVtbl->Release(properties);
	/* Read the properties, and then get rid of them */
	
	IWiaVideo * video = NULL;
	if(!SUCCEEDED(CoCreateInstance(&CLSID_WiaVideo, NULL, CLSCTX_INPROC_SERVER, &IID_IWiaVideo, (void * *)&video))) return 0;
	/* Make a video object */
	
	if(!SUCCEEDED(video->lpVtbl->put_ImagesDirectory(video, variant.bstrVal)))
	{
		video->lpVtbl->Release(video);
		return 0;
	}
	/* Set the image directory to the default.  For some reason it refuses to
	   work unless you explicitly set the image directory, even if it's just
	   the driver's default. */
	   
	if(!SUCCEEDED(video->lpVtbl->CreateVideoByWiaDevID(video, id, NULL, FALSE, FALSE)))
	{
		video->lpVtbl->Release(video);
		return 0;
	}
	if(!SUCCEEDED(video->lpVtbl->Play(video)))
	{
		video->lpVtbl->Release(video);
		return 0;
	}
	/* Open the video window */
	
	return (int)video;
}

JNIEXPORT void JNICALL Java_jwia_JWIA_closeCamera(JNIEnv * env, jclass j_class, jint video_int)
{
	IWiaVideo * video = (IWiaVideo *)video_int;
	video->lpVtbl->Release(video);
}

JNIEXPORT jstring JNICALL Java_jwia_JWIA_takeSnapshot(JNIEnv * env, jclass j_class, jint video_int)
{
	IWiaVideo * video = (IWiaVideo *)video_int;
	BSTR name;
	unsigned int i;
	
	if(!SUCCEEDED(video->lpVtbl->TakePicture(video, &name))) return NULL;
	/* Take the snapshot */
	
	size_t buffer_size = wcstombs(NULL, name, 0);
	char * buffer = (char *)malloc(buffer_size);
	for(i = 0; i < buffer_size; i++) buffer[i] = name[i];
	/* Copy the wide-char filename into a char array.  This loses the upper 8
	   bits of each character, but Java won't let us use the Unicode bits
	   in JNI anyway. */
	
	jstring return_me = (*env)->NewStringUTF(env, buffer);
	free(buffer);
	return return_me;
}
